/*= Rohde & Schwarz - Audio Analyzer UPL ====================================*/
#include "rsuplu.h"
#include "rsupl.h"
#include <formatio.h>
#include <userint.h>
#include <stdlib.h>
/* Needed if linking in external compiler; harmless otherwise */
#include <cvirte.h>

/*======================== Global Variabe Definition ========================*/ 
static ViInt32 panelHandle[10], uirCode;
static ViStatus uirErr;
static ViSession instrHandle;
static ViInt16 analyzerChannel = 0;

/*====================== Utility Function Declarations ======================*/ 
int abortApplication (void);

/*======================== Application Macros ===============================*/
#define appCheckErr(fCal)  if (uirErr = (fCal), uirErr < 0) {       \
				                if (uirErr >= 0xBFFF0000) {         \
				                    abortApplication ();            \
				                } else {                            \
				                    rsupl_error_message (VI_NULL, uirErr, errMsg); \
				                    MessagePopup ("Error", errMsg); \
				                    return 0;                       \
				                }                                   \
				            }


/*= Message Arrays ==========================================================*/
static char *initCtrlsHelpStr[] = {"This control selects address of the device that is to be initialized.\n\n"
                                   "Valid Range:  Address 0 to 30\n\n"
                                   "Default Value :  Address 20\n",
                                   "This control specifies if an ID Query is sent to the\n"
                                   "instrument during the initialization procedure.\n\n"
                                   "Valid Range:\n"
                                   "VI_FALSE (0) - Skip Query\n"
                                   "VI_TRUE  (1) - Do Query (Default Value)\n",
                                   "This control specifies if the instrument is to be reset to its\n"
                                   "power-on settings during the initialization procedure.\n\n"
                                   "Valid Range:\n"
                                   "VI_FALSE (0) - Don't Reset\n"
                                   "VI_TRUE  (1) - Reset Device (Default Value)\n"};  
    
static char *configGenCtrlsHelpStr[] = {"\nThis control selects the active channel\n"
                                       "of the digital generator on which the generated\n"
                                       "signal will be output. The signal\n"
                                       "will be available on the XLR conector\n"
                                       "of the corresponding channel.\n\n"
                                       "Valid Range:\n\n"
                                       "0 - Channel 1 (Default Value)\n"
                                       "1 - Channel 2\n",
                                       "\nThis control sets the frequency\n"
                                       "of the output sinewave signal.\n\n"
                                       "Valid Range: 10.0 Hz to 110.0 kHz\n\n"
                                       "Default Value: 1000.0 Hz\n",
                                       "\nThis control sets the amplitude of the output sinewave.\n\n"
                                       "Valid Range: 0.0 FS to 1.0 FS\n\n"
                                       "Default Value: 0.5 FS\n",
                                       "\nThis control sets the DC offset value.\n\n"
                                       "Valid Range: -1.0 FS to 1.0 FS\n\n"
                                       "Default Value: 0.0 FS\n"};

static char *configAnlCtrlsHelpStr[] = {"\nThis control selects the input channel\n"
                                        "of the digital analyzer at which the measured\n"
                                        "signal is applied.\n\n"
                                        "Valid Range:\n\n"
                                        "0 - Channel 1 (Default Value)\n"
                                        "1 - Channel 2\n",
                                        "\nThis control selects the input interface\n"
                                        "of the digital analyzer.\n\n"
                                        "Valid Range:\n\n"
                                        "0 - Bal (XLR) (Default Value)\n"
                                        "1 - Unbal (BNC)\n"
                                        "2 - Optical\n"
                                        "3 - Intern\n",
                                        "\nThis control specifies the display unit\n"
                                        "of the measured peak value.\n\n"
                                        "Valid Range:\n\n"
                                        "13 - dBr\n"
                                        "14 - FS (Default Value)\n"
                                        "15 - %FS\n"
                                        "16 - dBFS\n"
                                        "17 - LSBs\n"
                                        "18 - d%\n"
                                        "19 - bits\n"
                                        "20 - hex\n",
                                        "\nThis control sets the peak reference value.\n\n"
                                        "Valid Range:\n\n"
                                        "-100.0 FS to 100.0 FS\n\n"
                                        "Default Value: 1.0 FS\n",
                                        "\nThis control specifies the display unit\n"
                                        "of the measured frequency value.\n\n"
                                        "Valid Range:\n\n"
                                        "0 - Hz (Default Value)\n"
                                        "1 - dHz\n"
                                        "2 - d%Hz\n"
                                        "3 - Toct\n"
                                        "4 - Oct\n"
                                        "5 - Dec\n"
                                        "6 - f/fr\n",
                                        "\nThis control sets the reference frequency value.\n\n"
                                        "Valid Range: 1.0 mHz to 1.0 MHz\n\n"
                                        "Default Value: 0.0 Hz\n"};

static char *measCtrlsHelpStr[] =   {"\nThis control returns the result\n"
                                        "of the peak measurement\n"
                                        "at the specified instrument's channel.\n\n"
                                        "Note:\n\n"
                                        "If no measurement was taken,\n"
                                        "NAN (9.93E+37) is returned.\n",
                                        "\nThis control returns the result\n"
                                        "of the frequency measurement\n"
                                        "at the specified instrument's channel.\n\n"
                                        "Note:\n\n"
                                        "If no measurement was taken,\n"
                                        "NAN (9.93E+37) is returned.\n",
                                        "\nThis button will terminate the measurement.\n"};

static char backgroundPnlHelpStr[] = {"\nThis is a simple demo program using the Rohde&Schwarz UPL 10 Hz \n"
                                        "to 110 kHz Audio Analyzer instrument driver provided in the package.\n\n"
                                        "This example configures the instrument's internal generator and analyzer\n"
                                        "and allows the user to run a measurement. It is recommended that the user \n"
                                        "connect the generator's output to the analyzer's input to obtain representative\n"
                                        "measurement results. Nevertheless, the digital signal can be generated as well\n"
                                        "as analyzed be means of any other suitable external device.\n"};

static char initPnlHelpStr[] =      {"\nThis panel performs the following initialization actions:\n\n"
                                        "- Opens a session to the Default Resource Manager resource and a\n"
                                        "session to the specified device using the interface and address\n"
                                        "specified in the Resource_Name control.\n\n"
                                        "- Performs an identification query on the Instrument.\n\n"
                                        "- Resets the instrument to a known state.\n\n"
                                        "- Sends initialization commands to the instrument that set any\n"
                                        "necessary programmatic variables such as Headers Off, Short\n"
                                        "Command form, and Data Transfer Binary to the state necessary\n"
                                        "for the operation of the instrument driver.\n\n"
                                        "- Returns an Instrument Handle which is used to differentiate\n"
                                        "between different sessions of this instrument driver.\n\n"
                                        "- Each time this function is invoked a Unique Session is opened.\n"
                                        "It is possible to have more than one session open for the same\n"
                                        "resource.\n"};

static char configGenPnlHelpStr[] = {"\nThis panel performs a set of operations which\n"
                                        "completely configure the instrument's internal generator\n"
                                        "to output a digital audio sinewave signal.\n\n"
                                        "A call to Generator - Output Digital Sine function of the\n"
                                        "Rohde&Schwarz Audio Analyzer 10 Hz ... 110 kHz UPL\n"
                                        "is done when the \"Continue\" button is pressed.\n\n"
                                        "The Output Digital Sine function performs then\n"
                                        "the following operations:\n\n"
                                        "1) Resets the instrument\n"
                                        "2) Configures the instrument's trigger system\n"
                                        "3) Turns off the instrument's front panel display to speed up its operation\n"
                                        "4) Configures the internal digital generator\n"
                                        "5) Configures synchronization signals of the digital generator\n"
                                        "6) Sets up the generator function (sine)\n"
                                        "7) Triggers the instrument to start output of the desired signal\n"
                                        "8) Checks the instrument for an error\n"};

static char configAnlPnlHelpStr[] = {"\nThis panel performs a set of operations which\n"
                                        "completely configure the instrument's internal analyzer\n"
                                        "to take a RMS measurement at the input.\n\n"
                                        "A call to several configuration functions of the\n"
                                        "Rohde&Schwarz Audio Analyzer 10 Hz ... 110 kHz UPL\n"
                                        "is done when the \"Continue\" button is pressed.\n\n"
                                        "This panel performs the following operations:\n\n"
                                        "1) Configures the instrument's digital analyzer\n"
                                        "2) Configures the measurement output display\n"};

static char measPnlHelpStr[] =      {"\nThis panel performs measurement values readout.\n"
                                        "The measurement is triggered by pressing the\n"
                                        "\"Start Measurement\" button on this panel.\n"
                                        "Measured values are updated continuously\n"
                                        "until the \"Stop Measurement\" button is pushed.\n\n"
                                        "Configuration of the Rohde&Schwarz Audio Analyzer\n"
                                        "10 Hz ... 110 kHz UPL is not possible with\n"
                                        "this panel. Press the button \"Configure Generator\"\n"
                                        "or \"Configure Analyzer\" to return to configuration panel.\n"};

static char closePnlHelpStr[] = {"\nThis panel closes the instrument and takes it off-line.\n"};

/*= Main Function ===========================================================*/

void main ()
{
	panelHandle[BCKGRND] = LoadPanel (0, "rsuplu.uir", BCKGRND);
    panelHandle[MEAS] = LoadPanel (panelHandle[BCKGRND], "rsuplu.uir", MEAS);
    panelHandle[CONFIG_GEN] = LoadPanel (panelHandle[BCKGRND], "rsuplu.uir", CONFIG_GEN);
    panelHandle[CONFIG_ANL] = LoadPanel (panelHandle[BCKGRND], "rsuplu.uir", CONFIG_ANL);
    panelHandle[INIT] = LoadPanel (panelHandle[BCKGRND], "rsuplu.uir", INIT);
    DisplayPanel (panelHandle[BCKGRND]);
    DisplayPanel (panelHandle[INIT]);
    uirCode = RunUserInterface ();
}

/*===========================================================================*/
/* Function: Quit Application                                                */
/* Purpose:  This is a callback function of all Quit buttons of this         */
/*           application. It closes the instrument and quits the user        */
/*           interface.                                                      */
/*===========================================================================*/
int quitApplication (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    	{
        case EVENT_COMMIT:
            if ((ConfirmPopup ("Exit Application", "Would you like to exit this "
                    "application?")) == 1)
				{
                if ((uirErr = rsupl_close (instrHandle)) <0 )
                    abortApplication ();
                QuitUserInterface (uirCode);
            	}
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Control Help",
                  "Closes this session with instrument and closes the "
                  "application.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Initialize Instrument                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Initialize panel. It initializes the instrument and switches to */
/*           the panel Configure.                                            */
/*===========================================================================*/
int initInstrument (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 addr, rst, id;
    ViChar instrDescr[20];

    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[INIT], INIT_ADDRESS, &addr);
            GetCtrlVal (panelHandle[INIT], INIT_ID, &id);
            GetCtrlVal (panelHandle[INIT], INIT_RST, &rst);
            Fmt (instrDescr, "GPIB::%d[b2]", addr);
            if (rsupl_init (instrDescr, id, rst, &instrHandle))
            	{
                MessagePopup ("Error", "Initialization Error. Check your connections and make sure you have the right \n"
                              "                             GPIB address, and that the instrument is turned on.");
                return (0);            
            	}
            else
            	{
                HidePanel (panelHandle[INIT]);
                DisplayPanel (panelHandle[CONFIG_GEN]);
            	}
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button causes the instrument to be initialized.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls that configure the  */
/*           instrument. On the right mouse-click on the control a help      */
/*           window describing its purpose is displayed.                     */
/*===========================================================================*/
int controlHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK) {
        if (panel == panelHandle[INIT]) 
            MessagePopup ("Help", initCtrlsHelpStr[control-3]);
        else if (panel == panelHandle[CONFIG_GEN])      
            MessagePopup ("Help", configGenCtrlsHelpStr[control-2]);
        else if (panel == panelHandle[CONFIG_ANL])      
            MessagePopup ("Help", configAnlCtrlsHelpStr[control-3]);
        else if (panel == panelHandle[MEAS])                                 
            MessagePopup ("Help", measCtrlsHelpStr[control-6]);
    }
    return 0;
}

/*===========================================================================*/
/* Function: Configure Generator                                             */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Configure Generator panel. The instrument is configured using   */
/*           the Generator - Output Digital Sine  function of                */
/*           the Rohde&Schwarz Audio Analyzer 10 Hz ... 110 kHz UPL          */
/*           instrument driver.                                              */
/*===========================================================================*/
int configGenerator (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt16 gen_chan = 0;
    ViReal64 gen_freq = 1000.0;
    ViReal64 gen_ampl = 0.5;
    ViReal64 gen_offs = 0.0;
    ViChar errMsg[256];

    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[CONFIG_GEN], CONFIG_GEN_CHAN, &gen_chan);
            GetCtrlVal (panelHandle[CONFIG_GEN], CONFIG_GEN_FREQ, &gen_freq);
            GetCtrlVal (panelHandle[CONFIG_GEN], CONFIG_GEN_AMPL, &gen_ampl);
            GetCtrlVal (panelHandle[CONFIG_GEN], CONFIG_GEN_OFFS, &gen_offs);
            
            appCheckErr( rsupl_applicGenOutSine (instrHandle, gen_chan, gen_freq, gen_ampl, gen_offs));

            HidePanel (panelHandle[CONFIG_GEN]);
            DisplayPanel (panelHandle[CONFIG_ANL]);
            break;
            
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will configure the instrument's internal generator.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Configure Analyzer                                              */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Configure Analyzer panel. The instrument is configured using    */
/*           several functions of the Rohde&Schwarz Audio Analyzer           */
/*           10 Hz ... 110 kHz UPL instrument driver.                        */
/*===========================================================================*/
int configAnalyzer (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    short anl_peakUnit = 0;
    short anl_freqUnit = 0;
    short anl_inp = 0;
    double anl_peakRef = 0.0;
    double anl_freqRef = 0.0;
    char errMsg[256];

    switch (event)
    	{
        case EVENT_COMMIT:
            GetCtrlVal (panelHandle[CONFIG_ANL], CONFIG_ANL_CHAN, &analyzerChannel);
            GetCtrlVal (panelHandle[CONFIG_ANL], CONFIG_ANL_INP, &anl_inp);
            GetCtrlVal (panelHandle[CONFIG_ANL], CONFIG_ANL_PEAKUNIT, &anl_peakUnit);
            GetCtrlVal (panelHandle[CONFIG_ANL], CONFIG_ANL_FREQUNIT, &anl_freqUnit);
            GetCtrlVal (panelHandle[CONFIG_ANL], CONFIG_ANL_PEAKREF, &anl_peakRef);
            GetCtrlVal (panelHandle[CONFIG_ANL], CONFIG_ANL_FREQREF, &anl_freqRef);
            
            appCheckErr( rsupl_confAnalyzerDigStp (instrHandle, 0, analyzerChannel,
                                                   0, anl_inp, VI_FALSE,
                                                   48000.0, 3, 20));
            appCheckErr( rsupl_confAnalyzerFuncRMSSN (instrHandle, VI_FALSE, 3,
                                                      0.3, 13, 13, 0.0, VI_TRUE,
                                                      0, 1.0, 9, 0, 0, 10.0));
            appCheckErr( rsupl_confAnalyzerMeasInpDisp (instrHandle, 1,
                                                        anl_peakUnit, anl_peakUnit,
                                                        0, anl_peakRef, 9));
            appCheckErr( rsupl_confAnalyzerMeasInpFreq (instrHandle, 1,
                                                        anl_freqUnit, anl_freqUnit,
                                                        0, anl_freqRef, 0, 0.0, 0, 0));
            appCheckErr( rsupl_confInstrTrig (instrHandle, VI_TRUE));
            appCheckErr( rsupl_sendTrig (instrHandle));

            HidePanel (panelHandle[CONFIG_ANL]);
            DisplayPanel (panelHandle[MEAS]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will configure the instrument's internal analyzer.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void panelHelp (int menuBar, int menuItem, void *callbackData, int panel)
{
    if (panel == panelHandle[BCKGRND])
        MessagePopup ("Help", backgroundPnlHelpStr);
    if (panel == panelHandle[INIT])
        MessagePopup ("Help", initPnlHelpStr);
    if (panel == panelHandle[CONFIG_GEN])
        MessagePopup ("Help", configGenPnlHelpStr);
    if (panel == panelHandle[CONFIG_ANL])
        MessagePopup ("Help", configAnlPnlHelpStr);
    if (panel == panelHandle[MEAS])
        MessagePopup ("Help", measPnlHelpStr);
        
    return;
}

/*===========================================================================*/
/* Function: Start Measurement                                               */
/*           This is a callback function of the Start Measurement button on  */
/*           the panel Measure. It triggers a continuous measurement.        */
/*===========================================================================*/
int startMeasurement (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char errMsg[256];
    double val1 = 0.0;
    double val2 = 0.0;
    int ctrlVal = 0;
    int pnl, ctrl;
    
    switch (event) {
        case EVENT_COMMIT:
            while (!ctrlVal)
            	{
                appCheckErr( rsupl_measSingleCh (instrHandle, analyzerChannel, 1, &val1));
                appCheckErr( rsupl_measSingleCh (instrHandle, analyzerChannel, 2, &val2));
                SetCtrlVal (panelHandle[MEAS], MEAS_OUT_PEAK, val1);
                SetCtrlVal (panelHandle[MEAS], MEAS_OUT_FREQ, val2);
                ctrlVal = GetUserEvent (0, &pnl, &ctrl);
            	}
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will start the measurement.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Launch Panel Generator Config                                   */
/* Purpose:  This is a callback function of the button Configure Generator   */
/*           on the panel Measure. It calls the panel Configure Generator.   */
/*===========================================================================*/
int launchGenConfig (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (panelHandle[MEAS]);
            DisplayPanel (panelHandle[CONFIG_GEN]);
            SetActivePanel (panelHandle[CONFIG_GEN]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will open a panel to re-configure the instrument's internal generator.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Launch Panel Analyzer Config                                    */
/* Purpose:  This is a callback function of the button Configure Analyzer    */
/*           on the panel Measure. It calls the panel Configure Analyzer.    */
/*===========================================================================*/
int launchAnlConfig (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (panelHandle[MEAS]);
            DisplayPanel (panelHandle[CONFIG_ANL]);
            SetActivePanel (panelHandle[CONFIG_ANL]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will open a panel to re-configure the instrument's internal analyzer.");
            break;
    }
    return 0;
}

/*****************************************************************************/ 
int CVICALLBACK Cancel (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            if ((ConfirmPopup ("Exit Application", "Would you like to exit this "
                   "application?")) == 1)
				{
                QuitUserInterface (uirCode);
            	}
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Control Help",
                  "Closes this session with instrument and closes the application.");
            break;
    }
    return 0;
}

/*****************************************************************************/ 
int abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];

    SetWaitCursor (0);
    rsupl_error_message(VI_NULL, uirErr, errorMessage);
    Fmt (errBuffer,"%s<The following error caused the application to fail:\n\n%s\n\n"
        "Attempting to Close Instrument Driver and Exit Application", 
        errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
    
    if ((uirErr = rsupl_close (instrHandle)) < 0)
    	{
        rsupl_error_message(VI_NULL, uirErr, errorMessage);
        Fmt (errBuffer, "%s<Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    	}
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
        
    QuitUserInterface (uirCode);
    exit (-1);
    
    return 0;
}

/*= End =====================================================================*/
