/*****************************************************************************/
/*   Copyright 1997 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/

#include <formatio.h>
#include <utility.h>
#include <visa.h>
#include <stdlib.h>
#include <stdio.h>
#include "rsupl.h"

#define RSUPL_REVISION       "Rev 2.23.2, 11/2007, CVI 8.1." /*  Instrument driver revision */
#define BUFFER_SIZE          1024L                     /* File I/O buffer size        */
#define LONG_BUFFER_SIZE     16384L                    /* Long buffer size            */

/*===========================================================================*/
/*= Rohde & Schwarz - Audio Analyzer UPL ====================================*/
/* LabWindows/CVI 5.0.1 Instrument Driver                                    */
/* Original Release: April 1998                                              */
/* By: Michal Harhaj                                                         */
/*     Ph.+420 69 6994882  Fax +420 69 6919597  E-mail Michal.Harhaj@vsb.cz  */
/*                                                                           */
/* For LabWindows Technical Support in the United States call:               */
/*              National Instruments, Austin, Texas                          */
/*              Ph (800) 433-3488    Fax (512) 794-5678                      */
/*                                                                           */
/* Modification History:                                                     */
/*                                                                           */
/*           November 2007   Rev 2.23.2, 11/2006, CVI 8.1                    */
/*                           - Added rsupl_confAnalyzerSingleFilterSel       */
/*                           By: Jiri Kominek                                */
/*                                                                           */
/*             August 2006   Rev 2.23.1, 08/2006, CVI 8.0                    */
/*                           - New installer developed                       */
/*                           - Fixed Visual Basic Includes                   */
/*                           - Added C# wrapper                              */
/*                           By: Jiri Kominek                                */
/*                                                                           */
/*                May 2004   Rev 2.23, 05/2004, CVI 6.0                      */
/*                           Added CHM help file                             */
/*                           By: Jiri Kominek                                */   
/*                                                                           */
/*              March 2003   Rev 2.22, 03/2003, CVI 6.0                      */
/*                           Fixed rsupl_confGenFuncMult to accept limit of  */
/*                           240 chars per one command                       */  
/*                           By: Jiri Kominek                                */   
/*                                                                           */
/*          September 2002   Rev 2.21, 09/2002, CVI 6.0                      */
/*                           Added support for serial Interface              */
/*                           By: Werner Klotzsche, Rohde & Schwarz           */   
/*                                                                           */
/*              March 2000:  Rev 2.2, 03/2000, CVI 5.0.1                     */
/*                           - Changed data type for input string parameters */
/*                             from ViChar _VI_FAR xxx[] to ViString xxx.    */
/*                             It's important for HP-VEE                     */
/*                           By: Zdenek Rykala, ELCOM, a.s., Czech Republic  */
/*                                                                           */
/*            August  1999:  Fixed following functions:                      */
/*                           Total Harmonic Distortion                       */
/*                                                (rsupl_confAnalyzerFuncTHD)*/
/*                           Modulation Distorsion(rsupl_confGenFuncModDist) */
/*                           Random(rsupl_confGenFuncRnd)                    */
/*                           Analog Analyzer Setup(rsupl_confAnalyzerAnalStp)*/
/*                           Digital Analyzer Setup(rsupl_confAnalyzerDigStp)*/
/*                                                                           */
/*                           "Rev 2.1, 08/1999, CVI 5.0.1"                   */
/*                           Modified By: Zdenek Rykala, ELCOM, a.s.         */
/*                           Ph. +420 69 6996146   Fax. +420 69 6996147      */
/*                                                                           */
/*              July  1999:  Update to the latest VXIpnp standard.           */
/*                           "Rev 2.0, 07/1999, CVI 5.0.1"                   */
/*                           Modified By: Zdenek Rykala, ELCOM, a.s.         */
/*                           Ph. +420 69 6996146   Fax. +420 69 6996147      */
/*                                                                           */
/*              March 1998:  Originaly Release                               */
/*                           "Rev 1.0, 04/98, CVI 4.0.1"                     */
/*                           Modified By: Michal Harhaj, VSB-TU Ostrava      */
/*                                                                           */
/*===========================================================================*/

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString outTypeArr[] = {"UNB","BAL"};
static ViString outImpArr[] = {"R10","R200","R600"};
static ViString chanArr[] = {"OFF","CH1","CH2","CH2I"};
static ViString srcModeArr[] = {"ADAT","PHAS","JITT","COMM"};
static ViString unbOutArr[] = {"AOUT","AINP"};
static ViString cableSimulArr[] = {"OFF","SIML"};
static ViString inpSyncSourArr[] = {"GCL","AINP","RINP","SINP"};
static ViString inpSyncModeArr[] = {"WCL","IWCL","F1024","V50","V60"};
static ViString outSyncSourArr[] = {"AINP","GCL","RINP","SPLL"};
static ViString syncSigTypeArr[] = {"WCL","BCL"};
static ViString outDataArr[] = {"AZER","AONE"};
static ViString refOutArr[] = {"AINP","AINR","AOUT","RGEN"};
static ViString samplFreqArr[] = {"F32","F44","F48","EXT","SYNC","VAL"};
static ViString unitLevelArr[] = {"V","MV","UV","DBV","DBU","DBM","VVR","DPCTV","DV",
                    "DMV","DUV","VPP","MVPP","UVPP","DBR","FS","PCTFS","DBFS","LSBS",
                    "DPCT","BITS","UI","PCTUI","DBUI","PPMUI","NS","UIR"};
static ViString unitDCOffsArr[] = {"V","MV","UV","DBV","DBU","DBM","VVR","DPCTV","DV",
                    "DMV","DUV","DBR","FS","PCTFS","DBFS","LSBS","DPCT","BITS"};
static ViString unitBrsLowLevelArr[] = {"V","MV","UV","DBV","DBU","DBM","VVR","DPCTV",
                    "DV","DMV","DUV","VPP","MVPP","UVPP","VVON","PCTON","DBON","DBR",
                    "FS","PCTFS","DBFS","LSBS","BITS"};
static ViString unitJittRefLevelArr[] = {"UI","PCTUI","DBUI","PPMUI","NS","UIR","DBR"};
static ViString unitDFDRefLevelArr[] = {"V","MV","UV","DBV","DBU","UI","PCTUI","DBUI",
                    "PPMUI","NS","UIR","DBR","FS","PCTFS","DBFS","LSBS","DPCT","BITS"};
static ViString unitJittRndLevelArr[] = {"V","MV","UV","DBV","DBU","DBM","VVR","DPCTV",
                    "DV","DMV","DUV","DBR","FS","PCTFS","DBFS","LSBS","DPCT","BITS",
                    "UI","PCTUI","DBUI","PPM","NS","UIR","DBR"};
static ViString unitDigLevelArr[] = {"FS","PCTFS","DBFS","LSBS"};
static ViString chanAESArr[] = {"NONE","CH1A"};
static ViString chanStatusArr[] = {"ZERO","AES3","CRC","RAW","LEQ"};
static ViString userDataArr[] = {"ZERO","FILE"};
static ViString stateArr[] = {"OFF","ON"};
static ViString unitFreqArr[] = {"HZ","KHZ","DHZ","DKHZ","FFR","DPCTHZ","TOCT","OCT","DEC"};
static ViString PDFArr[] = {"GAUS","TRI","RECT"};
static ViString multisinModeArr[] = {"EQU","DEF"};
static ViString rndSpacingArr[] = {"USER","ATR"};
static ViString rndShpArr[] = {"WHIT","PINK","TOCT","ARB"};
static ViString generatorTypeArr[] = {"A25","D48"};
static ViString unitTimeArr[] = {"S","MS","US","CYC","KCYC","MCYC","MIN"};
static ViString unitIntervalArr[] = {"S","MS","US","MIN"};
static ViString levelTypeArr[] = {"LEV","LEV:RMS"};
static ViString sweFuncArr[] = {"FREQ","VOLT","ONT","INT"};
static ViString sweCtrlArr[] = {"FIX","SWE","LIST","SWE","LIST"};
static ViString sweOperArr[] = {"AUTO","AUTO","AUTO","MAN","MAN"};
static ViString sweSpacingArr[] = {"LIN;POIN","LOG;POIN","LIN;STEP","LOG;STEP"};
static ViString analyzerTypeArr[] = {"1","2","4"};
static ViString analyzerInputArr[] = {"CH1","CH2","CH1A","CH2I","CH1I"};
static ViString chan1InpArr[] = {"BAL","GEN2"};
static ViString chan2InpArr[] = {"BAL","GEN1"};
static ViString chanImpArr[] = {"R300","R600","R200K"};
static ViString chanComArr[] = {"FLO","GRO"};
static ViString chanRangModeArr[] = {"UPP","LOW"};
static ViString chanRangeArr[] = {"0.018","0.03","0.06","0.1","0.18","0.3","0.6","1.0",
                    "1.8","3.0","6.0","10.0","18.0","30.0"};
static ViString measModeArr[] = {"ADAT","JPH","CINP"};
static ViString digInpArr[] = {"AES","SPD","OPT","INT"};
static ViString digChanArr[] = {"CH1","CH2","BOTH"};
static ViString digSamplFreqArr[] = {"VAL","F32","F44","F48"};
static ViString digSyncSourArr[] = {"AINP","RINP"};
static ViString jitterRefArr[] = {"GCL","PLLV","PLL32","PLL44","PLL48"};
static ViString rmsMeasTimeArr[] = {"VAL","TRIG","AFAS","AUTO","GENT"};
static ViString rmsRefArr[] = {"VAL","CH1S","CH2S","STOR","CH1M","CH2M","GENT"};
static ViString referenceValUnitArr[] = {"V","MV","UV","DBV","DBU","DBM","W","MW",
                    "UW","FS","PCTFS","DBFS","LSBS","DPCT","BITS","DBR","UI","PCTUI",
                    "DBUI","PPMUI","NS","UIR"};
static ViString settlingResUnitArr[] = {"V","MV","UV","DBV","DBU","DBM","W","MW",
                    "UW","FS","PCTFS","DBFS","LSBS","DPCT","BITS","HZ","DEG","RAD"};
static ViString rmsNotchGainArr[] = {"OFF","DB0","DB12","DB30"};
static ViString rmsNotchFreqArr[] = {"FIX","GENT"};
static ViString triggerValUnitArr[] = {"PCT","DB"};
static ViString rmsUnitArr[] = {"V","DBV","DBU","DBM","W","DPCTV","DV","VVR",
                    "PCTVVR","DPCTW","DW","PPR","PCTPPR","FS","PCTFS","DBFS","LSBS",
                    "DPCT","BITS","DBR","UI","PCTUI","DBUI","PPMUI","NS","UIR","DB"};
static ViString settlTypeArr[] = {"OFF","EXP","FLAT","AVER"};
static ViString FFTWindowArr[] = {"STAT OFF","STAT ON;WIND HANN","STAT ON;WIND RECT",
                    "STAT ON;WIND BLAC","STAT ON;WIND RIF1","STAT ON;WIND RIF2",
                    "STAT ON;WIND RIF3","STAT ON;WIND HAMM","STAT ON;WIND FLAT",
                    "STAT ON;WIND KAIS"};
static ViString FFTSizeArr[] = {"S256","S512","S1K","S2K","S4K","S8K"};
static ViString rmsBWidthModeArr[] = {"PFIX","SFIX","PPCT1","PPCT3","PTOC","POCT1",
                    "SPCT1","SPCT3","STOC","SOCT1"};
static ViString rmsFreqModeArr[] = {"FIX","GENT","CH1","CH2"};
static ViString peakMeasModeArr[] = {"PPE","NPE","PTOP","PABS"};
static ViString intervalTimeArr[] = {"VAL","SFAS","FAST","SLOW","FIX"};
static ViString rmsSweCtrlArr[] = {"FIX","SWE;:SENS:SWE:MODE AUTO",
                    "LIST;:SENS:SWE:MODE AUTO","SWE;:SENS:SWE:MODE MAN",
                    "LIST;:SENS:SWE:MODE MAN","MULT"};
static ViString dcMeasTimeArr[] = {"VAL","FAST"};
static ViString thdMeasModeArr[] = {"SEL","DEV","DODD","DALL","LSEL","LDEV","LDOD","LDAL"};
static ViString thdDynModeArr[] = {"PREC","FAST"};
static ViString thdFundModeArr[] = {"AUTO","VAL"};
static ViString thdUnitNewArr[] = {"PCT","DB"};
static ViString thdUnitArr[] = {"V","DBV","DBU","DBM","W","DPCTV","DV","VVR","PCTVVR",
                    "DPCTW","DW","PPR","PCTPPR","FS","PCTFS","DBFS","LSBS","DPCT",
                    "BITS","DBR","PCT","DB"};
static ViString thdnMeasModeArr[] = {"THDN","SNDR","NOIS","LTHD","LNOI"};
static ViString thdnRejectionArr[] = {"NARR","WIDE"};
static ViString dfdMeasModeArr[] = {"D2_268","D2_118","D3_268","D3_118"};
static ViString wfStandardArr[] = {"DIN","NAB","JIS","SI05","SI10"};
static ViString funcFFTWindowArr[] = {"HANN","RECT","BLAC","RIF1","RIF2","RIF3",
                    "HAMM","FLAT","KAIS"};
static ViString FFTAverModeArr[] = {"EXP","NORM"};
static ViString FFTZoomArr[] = {"1","2","4","8","16","32","64","128"};
static ViString wfmMeasModeArr[] = {"STAN","COMP"};
static ViString trigSlopeArr[] = {"POS","NEG"};
static ViString smoothingArr[] = {"N1","N2","N4","N8","N16","N32"};
static ViString trigSourceArr[] = {"CH1","CH2","GENB"};
static ViString filterArr[] = {"UFIL1","UFIL2","UFIL3","UFIL4","UFIL5","UFIL6","UFIL7",
                    "UFIL8","UFIL9","AWE","CMES","CCIT","CCIR","CCIU","DEMP5015",
                    "DEMP50","DEMP75","DEMP17","WRUM","URUM","DCN","CARM","IECT","JITT"};
static ViString filterTypeArr[] = {"LPAS","HPAS","BPAS","BSTO","NOTC","TOCT","OCT","FILE"};
static ViString inpDispUnitArr[] = {"V","DBV","DBU","DBM","W","DPCTV","DV","VVR",
                    "PCTVVR","DPCTW","DW","PPR","PCTPPR","DBR","FS","PCTFS","DBFS",
                    "LSBS","DPCT","BITS","HEX","UI","PCTUI","PPMUI","NS","UIR","DBUI",
                    "PCTFRM","DEGFRM"};
static ViString inpDispModeArr[] = {"OFF","PEAK","RMS","PHAS","DIGI"};
static ViString inpDispUnitFreqArr[] = {"HZ","DHZ","DPCTHZ","TOCT","OCT","DEC","FFR",
                    "DEG","RAD","DDEG","DRAD","S","DS"};
static ViString inpDispRefUnitFreqArr[] = {"DEG","RAD","S","MS","US","MIN"};
static ViString inpDispFreqModeArr[] = {"OFF","FREQ","FQPH","FQGR","SFRE"};
static ViString inpPhasRefArr[] = {"VAL","STOR"};
static ViString phasFormatArr[] = {"POS","POSN","NEG","RAD","RADB","RADN"};
static ViString protocolCtrlArr[] = {"OFF","STAT","ENH"};
static ViString dfdGenModeArr[] = {"IEC118","IEC268"};
static ViString DFDPhasOfuplnitArr[] = {"UI","PCTFRM","DEGFRM","NS"};
static ViString stpModeArr[] = {"STOR","LOAD"};
static ViString stpTypeArr[] = {"0","2"};
static ViString settlParamArr[] = {"1:TRIG","1:FUNC","3:FREQ","3:PHAS"};
static ViString storeParamArr[] = {"TRAC TRAC1","TRAC TRAC2","TRAC TR1A","LIST ERR",
                    "LIST DWEL","LIST LIMU","LIST LIML","LIST LIST1","LIST LIST2",
                    "LIST EQU"};
static ViString dataFormatArr[] = {"ASC","BIN"};
static ViString dataLevSourceArr[] = {"TRAC1","TRAC2"};
static ViString stepDirArr[] = {"1","-1"};
static ViString blockReadPointsVarArr[] = {"SOUR:LIST:FREQ:POIN?",
                    "SOUR:LIST:VOLT:POIN?","SOUR:LIST:ONT:POIN?","SOUR:LIST:INT:POIN?",
                    "SOUR:LIST:DWEL:POIN?","SOUR:VOLT:EQU:POIN?","SOUR:EQU:CONT:POIN?",
                    "CALC:LIM:UPP:POIN?","CALC:LIM:LOW:POIN?", "CALC:LIM:UPP:CONT:POIN?",
                    "CALC:LIM:LOW:CONT:POIN?","TRAC:POIN? TRAC1","TRAC:POIN? TRAC2",
                    "TRAC:POIN? LIST1","TRAC:POIN? LIST2","TRAC:POIN? REF1",
                    "TRAC:POIN? REF2","TRAC:POIN? CREF1","TRAC:POIN? CREF2",
                    "CALC:LIM:REP:POIN?"};
static ViString blockReadVarArr[] = {"SOUR:LIST:FREQ?","SOUR:LIST:VOLT?",
                    "SOUR:LIST:ONT?","SOUR:LIST:INT?","SOUR:LIST:DWEL?",
                    "SOUR:VOLT:EQU?","SOUR:EQU:CONT?","CALC:LIM:UPP?","CALC:LIM:LOW?",
                    "CALC:LIM:UPP:CONT?","CALC:LIM:LOW:CONT?",
                    "TRAC? TRAC1","TRAC? TRAC2","TRAC? LIST1","TRAC? LIST2",
                    "TRAC? REF1","TRAC? REF2","TRAC? CREF1","TRAC? CREF2",
                    "CALC:LIM:REP?"};
static ViString blockWriteVarArr[] = {"SOUR:LIST:FREQ","SOUR:LIST:VOLT",
                    "SOUR:LIST:ONT","SOUR:LIST:INT","SOUR:LIST:DWEL","SOUR:VOLT:EQU",
                    "SOUR:EQU:CONT","CALC:LIM:UPP","CALC:LIM:LOW","CALC:LIM:UPP:CONT",
                    "CALC:LIM:LOW:CONT", "TRAC TRAC1,","TRAC TRAC2,","TRAC LIST1,",
                    "TRAC LIST2,","TRAC REF1,","TRAC REF2,", "TRAC CREF1,","TRAC CREF2,"};
static ViString calibArr[] = {"OFF","ONCE","ON"};   
static ViString hcopyDestArr[] = {"PCXF","HPGL"};
static ViString checkArr[] = {"TRAC1","TRAC2","TR1A"};
static ViString limUnitArr[] = {"V","MV","UV","DBV","DBU","DBM","VVR","DPCTV","DV",
                    "W","DW","DPCTW","PCTVVR","PCTPPR","PPR","FS","PCTFS","DBFS",
                    "DPCT","LSBS","DBR","BITS","UI","PCTUI","DBUI","PPMUI","NS",
                    "UIR","DB","PCT"};
static ViString protAnlSourceArr[] = {"LCH","RCH","LUS","RUS"};
static ViString protAnlFormatArr[] = {"BIN","HEX","ASC","FILE"};
static ViString speakerSourceArr[] = {"OFF","INP1","INP2","IN1A","FNC1","FNC2","FN1A",
                                      "AES1","AES2","AE1A"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rsupl_stringValPair is used in the rsupl_errorMessage function            */
/* rsupl_statusDataRanges is used to track session dependent status & ranges */
/* buffer is used to write to and to read from the instrument                */
/* longBuffer is used to write block data to and to read block data from     */
/*      the instrument                                                       */
/*===========================================================================*/
static ViByte buffer[BUFFER_SIZE];
static ViByte longBuffer[LONG_BUFFER_SIZE];

typedef struct  rsupl_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsupl_tStringValPair;

struct rsupl_statusDataRanges {
    ViChar instrDriverRevision[256];
};

typedef struct rsupl_statusDataRanges *rsupl_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rsupl_invalidViBooleanRange (ViBoolean val);
ViBoolean rsupl_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsupl_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rsupl_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
                            ViStatus currentStatus);

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS =======================*/
/*****************************************************************************/
ViStatus rsupl_defaultInstrSetup (ViSession openInstrSession);
ViStatus rsupl_sysStatus (ViSession instrSession);
ViStatus rsupl_confOPT (ViSession instrSession, ViString optsList);
ViStatus rsupl_getGenMode (ViSession instrSession, ViPBoolean InstrType,
                           ViPBoolean Mode);
ViStatus rsupl_getAnalyzerMode (ViSession instrSession, ViPInt32 digMode,
                                ViPBoolean instrType);
ViStatus rsupl_getAnalyzerActiveChannels (ViSession instrSession, ViPInt32 chanType);
ViStatus rsupl_getInstrResponse (ViSession instrSession, ViChar _VI_FAR query[],
                                 ViChar _VI_FAR resp[]);

/*****************************************************************************/
/*= USER-CALLABLE FUNCTIONS (Exportable Functions) ==========================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_init (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt16 interface = VI_INTF_GPIB;
    ViUInt32 retCnt = 0;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rsupl_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    rsupl_status =  viOpenDefaultRM (&rmSession);
    
    if ((rsupl_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG, VI_NULL,
        instrSession)) < 0) {
        viClose (rmSession);
        return rsupl_status;
    }

    /* Serial interface support */
    if ((rsupl_status = viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &interface)) < 0)
        return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);

    if (interface == VI_INTF_ASRL) /* Serial Port */
        {
        if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n')) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
        if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
        if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,
            VI_ASRL_END_TERMCHAR)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
        if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,
            VI_ASRL_END_TERMCHAR)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
        if ((rsupl_status = viFlush (*instrSession, VI_ASRL_IN_BUF)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
        if ((rsupl_status =  viVPrintf (*instrSession, "\n", 0)) < 0)
             return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);    
                  
        }


    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 60000)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
    if ((rsupl_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
    if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
    if ((rsupl_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) {
        if ((rsupl_status = viWrite (*instrSession, (ViByte *)"*IDN?", 5, &retCnt)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
        if ((rsupl_status = viRead (*instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rsupl_status;

        Scan (buffer, "ROHDE & SCHWARZ, UPL");
        if (NumFmtdBytes () != 20)
            return rsupl_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice) {
        if ((rsupl_status = rsupl_reset (*instrSession)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        if ((rsupl_status = rsupl_defaultInstrSetup (*instrSession)) < 0)
            return rsupl_initCleanUp (rmSession, instrSession, rsupl_status);
    
    return rsupl_status;
}

/*===========================================================================*/

/*****************************************************************************/
/*= USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES =============================*/
/*****************************************************************************/
/*===========================================================================*/
/* Function: Generator - Output Digital Sine                                 */
/* Purpose: This function demonstrates higher-level approach to the          */
/* instrument driver. It performs a set of operations which completely       */
/* configure the instrument to satisfy the user's needs. This function should*/ 
/* be used as an example for building applications using this instrument     */
/* driver.                                                                   */
/* This function performes the following operations:                         */
/* 1) resets the instrument                                                  */
/* 2) configures the instrument's trigger system                             */
/* 3) turns off the instrument's front panel display to speed up its         */
/*    operation                                                              */
/* 4) configures the internal digital generator                              */
/* 5) configures synchronization signals of the digital generator            */
/* 6) sets up the generator function (sine)                                  */
/* 7) triggers the instrument to start output of the desired signal          */
/* 8) checks the instrument for an error                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_applicGenOutSine (ViSession instrSession,
                                          ViBoolean channel, ViReal64 frequencyHz,
                                          ViReal64 levelV, ViReal64 DCOffsetV)
{
    ViStatus rsupl_status = VI_SUCCESS;

    if (rsupl_invalidViInt32Range (channel, 0, 1))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (frequencyHz, 10.0, 110.0E3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (levelV, 0.0, 10.0))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViReal64Range (DCOffsetV, -10.0, 10.0))
        return VI_ERROR_PARAMETER5;

    /* Configure analog generator --------------------------------------------*/
    if ((rsupl_status = rsupl_confGenAnStp (instrSession, channel+1, VI_FALSE, 0, 12.0,
                                1000.0, 1.0, 0, 0)) < 0)
        return rsupl_status;

    /* Configure generated sine signal ---------------------------------------*/
    if ((rsupl_status = rsupl_confGenFuncSin (instrSession, VI_FALSE,
                                              frequencyHz, 0, VI_TRUE, DCOffsetV,
                                              0, levelV, 0, VI_FALSE, "*.veq",
                                              VI_FALSE, 0.0, 15, 0, VI_FALSE)) < 0)
        return rsupl_status;

    /* Check instrument status -----------------------------------------------*/
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Analog Generator Setup                                          */
/* Purpose:  This function configures the following basic settings of the    */
/* analog generator:                                                         */
/* - active channels                                                         */
/* - output coupling                                                         */
/* - output impedance                                                        */
/* - output level limit                                                      */
/* - output level limit unit                                                 */
/* - reference frequency                                                     */
/* - reference level                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenAnStp (ViSession instrSession, ViInt32 channel,
    ViBoolean output, ViInt32 outImp, ViReal64 voltLimit, ViReal64 refFreq,
    ViReal64 refVolt, ViInt32 voltLimitUnit, ViInt32 refVoltUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (channel, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (output))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (outImp, 0, 2))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (voltLimitUnit, 0, 4))
        return VI_ERROR_PARAMETER8;
    if (voltLimitUnit == 0)                   
        if (rsupl_invalidViReal64Range (voltLimit, 0.0, 20.0))
            return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViReal64Range (refFreq, 0.001, 1.0E6))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (refVoltUnit, 0, 4))
        return VI_ERROR_INSTR_PARAMETER9;
    if (refVoltUnit == 0)
        if (rsupl_invalidViReal64Range (refVolt, 1.0E-6, 1.0E6))
            return VI_ERROR_PARAMETER7;

    Fmt (buffer, "%s<:INST A25;:OUTP:TYPE %s;:SOUR:VOLT:LIM %f%s;REF %f%s;:SOUR:FREQ:REF %f;:OUTP:SEL %s",
        outTypeArr[output], voltLimit, unitLevelArr[voltLimitUnit], 
        refVolt, unitLevelArr[refVoltUnit], refFreq, chanArr[channel]);
    bytesFmt = NumFmtdBytes ();

    if (output == VI_TRUE) {  
        Fmt (buffer, "%s[a]<;:OUTP:IMP %s", outImpArr[outImp]);
        bytesFmt += NumFmtdBytes ();
    }       
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
                
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Digital Generator Setup                                         */
/* Purpose:  This function configures the following basic settings of the    */
/* digital generator:                                                        */
/* - source mode                                                             */
/* - active channels                                                         */
/* - audio bits                                                              */
/* - unbalanced output connector coupling                                    */
/* - phase offset                                                            */
/* - output level limit                                                      */
/* - output level limit unit                                                 */
/* - reference level                                                         */
/* - reference level unit                                                    */
/* - reference frequency                                                     */
/* - unbalanced level                                                        */
/* - unbalanced level unit                                                   */
/* - cable simulator                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenDigStp (ViSession instrSession, ViInt32 srcMode,
    ViInt32 channel, ViReal64 phasOffset, ViBoolean unbOut, ViBoolean cableSimul,
    ViReal64 voltLimit, ViReal64 refFreq, ViReal64 refVolt, ViInt32 audioBits,
    ViReal64 unbalVolt, ViInt32 unbalLevelUnit, ViInt32 limitLevelUnit, 
    ViInt32 refLevelUnit, ViInt32 phasOffsetUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (srcMode, 0, 3)) 
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (channel, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (phasOffsetUnit, 0, 4))          
        return VI_ERROR_INSTR_PARAMETER15;
    if (unbalLevelUnit == 0)        
        if (rsupl_invalidViReal64Range (phasOffset, -64.0, 64.0))
            return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViBooleanRange (unbOut))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViBooleanRange (cableSimul))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (limitLevelUnit, 14, 20))          
        return VI_ERROR_INSTR_PARAMETER13;
    if (limitLevelUnit == 0)
        if (rsupl_invalidViReal64Range (voltLimit, 0.0, 1.0))
            return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViReal64Range (refFreq, 0.001, 1.0E6))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (refLevelUnit, 0, 17))
        return VI_ERROR_INSTR_PARAMETER14;
    if (refLevelUnit == 0)  
        if (rsupl_invalidViReal64Range (refVolt, 0.0, 1.0E9))
            return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (audioBits, 8, 24))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (unbalLevelUnit, 0, 4))          
        return VI_ERROR_INSTR_PARAMETER12;
    if (unbalLevelUnit == 0)            
        if (rsupl_invalidViReal64Range (unbalVolt, 0.0, 2.125))
            return VI_ERROR_INSTR_PARAMETER11;
        
    Fmt (buffer, "%s<:INST D48;:SOUR:DIG:FEED %s;:OUTP:DIG:CSIM %s;UNB:FEED %s;:SOUR:FREQ:REF %f;:OUTP:SIGN:LEV %f%s;:SOUR:VOLT:REF %f%s",
        srcModeArr[srcMode], cableSimulArr[cableSimul], unbOutArr[unbOut], 
        refFreq, unbalVolt, unitLevelArr[unbalLevelUnit], refVolt, unitDFDRefLevelArr[refLevelUnit]);
    bytesFmt = NumFmtdBytes ();
    if (srcMode == 1) {
        Fmt (buffer, "%s[a]<;:SOUR:DIG:SYNC:DEL %f%s", phasOffset, DFDPhasOfuplnitArr[phasOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    if (srcMode < 2) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:LIM %f%s;:OUTP:AUD %d[b4]", voltLimit, unitLevelArr[limitLevelUnit], audioBits);
        bytesFmt += NumFmtdBytes ();
    }
    if (srcMode != 3) {
        Fmt (buffer, "%s[a]<;:OUTP:SEL %s", chanArr[channel]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Digital Generator Sync Signals                                  */
/* Purpose:  This function configures the digital generator's synchronization*/
/* settings:                                                                 */
/* - output synchronization source                                           */
/* - input synchronization source                                            */
/* - input synchronization mode                                              */
/* - sampling frequency                                                      */
/* - synchronization signal type                                             */
/* - output data                                                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenDigSync (ViSession instrSession, ViInt32 outSyncSour,
    ViInt32 inpSyncSour, ViBoolean syncSigType, ViInt32 refOut, ViBoolean outData,
    ViInt32 samplFreq, ViReal64 samplFreqVal, ViInt32 inpSyncMode)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (outSyncSour, 0, 3)) /* out sync source */
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (inpSyncSour, 0, 3)) /* sync to */
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (syncSigType)) /* sync signal type */
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (refOut, 0, 3))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViBooleanRange (outData))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (samplFreq, 0, 5))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViReal64Range (samplFreqVal, 30.0E3, 52.5E3))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (inpSyncMode, 0, 4))
        return VI_ERROR_INSTR_PARAMETER9;

    Fmt (buffer, "%s<:INST D48;:SOUR:DIG:SYNC:SOUR %s;:OUTP:DIG:SYNC:FEED %s;TYPE %s;:OUTP:DIG:REF:FEED %s;:OUTP:SAMP:MODE %s",
        inpSyncSourArr[inpSyncSour], outSyncSourArr[outSyncSour], 
        syncSigTypeArr[syncSigType], refOutArr[refOut],
        samplFreqArr[samplFreq]);
    bytesFmt = NumFmtdBytes ();
    
    if (samplFreq == 5) {
        Fmt (buffer, "%s[a]<;:OUTP:SAMP:FREQ %f", samplFreqVal);
        bytesFmt += NumFmtdBytes ();
    }
    if (inpSyncSour == 3) {
        Fmt (buffer, "%s[a]<;:SOUR:DIG:SYNC:MODE %s", inpSyncModeArr[inpSyncMode]);
        bytesFmt += NumFmtdBytes ();
    }
    if (refOut == 3) {
        Fmt (buffer, "%s[a]<;:SOUR:DIG:REF %s", outDataArr[outData]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Protocol                                                        */
/* Purpose:  This function configures the following AES/EBU protocol         */
/* definition settings:                                                      */
/* - valid channel                                                           */
/* - protocol mode                                                           */
/* - chan status data left                                                   */
/* - filename left                                                           */
/* - chan status data right                                                  */
/* - filename right                                                          */
/* - user data mode                                                          */
/* - filename user data                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenDigProtocol (ViSession instrSession, ViBoolean channel,
    ViInt32 chanStatusL, ViString fileNameL, ViString fileNameR, 
    ViBoolean userData, ViString fileNameUsrData, ViInt32 protocol, 
    ViInt32 chanStatusR)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
 
    if (rsupl_invalidViBooleanRange (channel))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (chanStatusL, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (userData))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (protocol, 0, 2))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (chanStatusR, 0, 4))
        return VI_ERROR_INSTR_PARAMETER9;
 
    if (protocol == 0) {
        Fmt (buffer, "%s<:SOUR:PROT %s", protocolCtrlArr[protocol]);
        bytesFmt = NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s<:SOUR:PROT %s;:SOUR:PROT:LCH %s", protocolCtrlArr[protocol], 
            chanStatusArr[chanStatusL]);
        bytesFmt = NumFmtdBytes ();
        if (chanStatusL != 0) {
            Fmt (buffer, "%s[a]<;:MMEM:LOAD:LPGC \"%s\"", fileNameL);
            bytesFmt += NumFmtdBytes ();
        }
        if (protocol == 2) {
            Fmt (buffer, "%s[a]<;:OUTP:VAL %s;:SOUR:PROT:RCH %s;UMOD %s",
                chanAESArr[channel], chanStatusArr[chanStatusR], userDataArr[userData]);
            bytesFmt += NumFmtdBytes ();
            if (chanStatusR != 0) {
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:RPGC \"%s\"", fileNameR);
                bytesFmt += NumFmtdBytes ();
            }
            if (userData == VI_TRUE) {
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:PGU \"%s\"", fileNameUsrData);
                bytesFmt += NumFmtdBytes ();
            }
        }
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Sine                                                            */
/* Purpose:  This function configures the generator to output a sine signal. */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncSin (ViSession instrSession,
                                        ViBoolean freqOffset, ViReal64 frequency,
                                        ViInt32 freqUnit, ViBoolean DCOffset,
                                        ViReal64 DCOffsetValue,
                                        ViInt32 DCOffsetUnit, ViReal64 level,
                                        ViInt32 levelUnit, ViBoolean equalizer,
                                        ViString fileNameEqualizer,
                                        ViBoolean dither, ViReal64 ditherValue,
                                        ViInt32 ditherUnit, ViInt32 PDF,
                                        ViBoolean lowDistortionGen)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean mode = VI_FALSE, instrType;

    if (rsupl_invalidViBooleanRange (freqOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
            return VI_ERROR_PARAMETER4;
    if (freqUnit == 0)
        if (rsupl_invalidViReal64Range (frequency, 10.0, 110.0E3))
            return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViBooleanRange (equalizer))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViBooleanRange (dither))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (ditherUnit, 14, 20))
        return VI_ERROR_INSTR_PARAMETER14;
    if (ditherUnit == 15)
        if (rsupl_invalidViReal64Range (ditherValue, 0.0, 1.0))
            return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViInt32Range (PDF, 0, 2))
        return VI_ERROR_INSTR_PARAMETER15;
    if (rsupl_invalidViBooleanRange (lowDistortionGen))
        return VI_ERROR_INSTR_PARAMETER16;

    if ((rsupl_status = rsupl_getGenMode (instrSession, &instrType, &mode)) < 0)
            return rsupl_status;
    if (instrType == VI_TRUE) { // digital
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 11, 17))
            return VI_ERROR_PARAMETER7;
        if (DCOffsetUnit == 12) {
            if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
                return VI_ERROR_PARAMETER6;
        }
        if (rsupl_invalidViInt32Range (levelUnit, 14, 26))
            return VI_ERROR_INSTR_PARAMETER9;
        if (levelUnit == 15) {
            if (rsupl_invalidViReal64Range (level, 0.0, 1.0))
                return VI_ERROR_PARAMETER8;
        }
        
        if (mode == VI_TRUE) {
            Fmt (buffer, "%s<:SOUR:FREQ:OFFS:STAT %s;:SOUR:FUNC:SHAP SIN;"
                 ":SOUR:FREQ %f%s;:SOUR:VOLT %f%s;:MMEM:LOAD:LIST EQU, \"%s\";"
                 ":SOUR:VOLT:EQU:STAT %s;:SOUR:SIN:DITH:STAT %s;:SOUR:VOLT:OFFS:STAT %s",
                 stateArr[freqOffset], frequency, unitFreqArr[freqUnit], level,
                 unitLevelArr[levelUnit], fileNameEqualizer, stateArr[equalizer],
                 stateArr[dither], stateArr[DCOffset]);
            bytesFmt = NumFmtdBytes ();
            if (DCOffset == VI_TRUE) {
                Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue,
                     unitDCOffsArr[DCOffsetUnit]);
                bytesFmt += NumFmtdBytes ();
            }           
            if (dither == VI_TRUE) {
                Fmt (buffer, "%s[a]<;:SOUR:SIN:DITH %f%s;:SOUR:RAND:PDF %s", 
                    ditherValue, unitLevelArr[ditherUnit], PDFArr[PDF]);
                bytesFmt += NumFmtdBytes ();
            }
        } else {
            Fmt (buffer, "%s<:SOUR:FUNC:SHAP SIN;:SOUR:FREQ %f%s;:SOUR:VOLT %f%s",
                frequency, unitFreqArr[freqUnit], level, unitLevelArr[levelUnit]);
            bytesFmt = NumFmtdBytes ();
        }
    } else {  // analog
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 11))
            return VI_ERROR_PARAMETER7;
        if (DCOffsetUnit == 0) {
            if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
                return VI_ERROR_PARAMETER6;
        }
        if (rsupl_invalidViInt32Range (levelUnit, 0, 14))
            return VI_ERROR_INSTR_PARAMETER9;
        if (levelUnit == 0) {
            if (rsupl_invalidViReal64Range (level, 0.0, 20.0))
                return VI_ERROR_PARAMETER8;
        }
        
        Fmt (buffer, "%s<:SOUR:FREQ:OFFS:STAT %s;:SOUR:FUNC:SHAP SIN;"
                    ":SOUR:LOWD %s;:SOUR:FREQ %f%s;:SOUR:VOLT %f%s;"
                    ":MMEM:LOAD:LIST EQU, \"%s\";:SOUR:VOLT:EQU:STAT %s",
            stateArr[freqOffset], stateArr[lowDistortionGen], frequency,
            unitFreqArr[freqUnit], level, unitLevelArr[levelUnit], fileNameEqualizer,
            stateArr[equalizer]);
        bytesFmt = NumFmtdBytes ();

        if (lowDistortionGen == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS:STAT %s", stateArr[DCOffset]);
            bytesFmt += NumFmtdBytes ();
            if (DCOffset == VI_TRUE) {
                Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue,
                     unitDCOffsArr[DCOffsetUnit]);
                bytesFmt += NumFmtdBytes ();
            }           
        }
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}        

/*===========================================================================*/
/* Function: Multisine                                                       */
/* Purpose:  This function configures the generator to output a multisine    */
/*           signal.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncMult (ViSession instrSession,
                    ViReal64 totalGaindB, ViInt32 numbOfWfms,
                    ViBoolean multisineMode,
                    ViReal64 _VI_FAR levelArray[],
                    ViInt32 levelUnit, ViBoolean spacing,
                    ViReal64 spacingValueHz,
                    ViReal64 _VI_FAR frequencyArray[],
                    ViInt32 freqUnit, ViBoolean DCOffset,
                    ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit, ViBoolean equalizer,
                    ViString fileNameEqualizer)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViInt32 i = 0;

    if (rsupl_invalidViReal64Range (totalGaindB, -80.0, 30.0))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (numbOfWfms, 1, 17))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (multisineMode))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (levelUnit, 0, 20))
        return VI_ERROR_PARAMETER6;
    if (levelUnit == 15)
        for (i = 0; i < numbOfWfms; i++) {
            if (rsupl_invalidViReal64Range (levelArray[i], 0.0, 1.0))
                return VI_ERROR_PARAMETER5;
        }
    if (levelUnit == 0)
        for (i = 0; i < numbOfWfms; i++) {
            if (rsupl_invalidViReal64Range (levelArray[i], 0.0, 20.0))
                return VI_ERROR_PARAMETER5;
        }
    if (rsupl_invalidViBooleanRange (spacing))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
        return VI_ERROR_INSTR_PARAMETER10;
    if (freqUnit == 0)
        for (i = 0; i < numbOfWfms; i++) {
            if (rsupl_invalidViReal64Range (frequencyArray[i], 10.0, 110.0E3))
                return VI_ERROR_INSTR_PARAMETER9;
        }
    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 17))
        return VI_ERROR_INSTR_PARAMETER13;
    if (DCOffsetUnit == 12) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
            return VI_ERROR_INSTR_PARAMETER12;
    }
    if (rsupl_invalidViBooleanRange (equalizer))
        return VI_ERROR_INSTR_PARAMETER14;
    
    Fmt (buffer, "%s<:SOUR:FUNC:SHAP MULT;:SOUR:VOLT:OFFS:STAT %s;"
                ":SOUR:MULT:MODE %s;:SOUR:VOLT:TOT:GAIN %f;:SOUR:MULT:COUN %d[b4];"
                ":SOUR:RAND:SPAC:MODE %s;:SOUR:VOLT:EQU:STAT %s",
        stateArr[DCOffset], multisinModeArr[multisineMode], 
        totalGaindB, numbOfWfms, spacing[rndSpacingArr], stateArr[equalizer]);
    
    bytesFmt = NumFmtdBytes ();

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    bytesFmt = 0;
    buffer[0] = '\0';

    for (i = 0; i < numbOfWfms; i++) {
        Fmt (buffer, "%s[a]<;:SOUR:FREQ%d[b4] %f%s", i + 1, frequencyArray[i], unitFreqArr[freqUnit]);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
        
    bytesFmt = 0;
    buffer[0] = '\0';

    if (multisineMode == VI_TRUE) {
        for (i = 0; i < numbOfWfms; i++) {
            Fmt (buffer, "%s[a]<;:SOUR:VOLT%d[b4] %f%s", i + 1, levelArray[i], unitLevelArr[levelUnit]);
            bytesFmt += NumFmtdBytes ();
        }
    } else {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT1 %f%s", levelArray[0], unitLevelArr[levelUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
        
    bytesFmt = 0;
    buffer[0] = '\0';
    
    if (DCOffset == VI_TRUE) {
        Fmt (buffer, "%s<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    if (spacing == VI_FALSE) { // user defined
        if (rsupl_invalidViReal64Range (spacingValueHz, 0.0, 110.0E3))
            return VI_ERROR_PARAMETER8;
        Fmt (buffer, "%s[a]<;:SOUR:RAND:SPAC:FREQ %f", spacingValueHz);
        bytesFmt += NumFmtdBytes ();
    }           
    if (equalizer == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST EQU, \"%s\"", fileNameEqualizer);
        bytesFmt += NumFmtdBytes ();
    }           
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Sine Burst                                                      */
/* Purpose:  This function configures the sine burst function generator. A   */
/*           sinewave periodically varying between high and low levels is    */
/*           output by the instrument.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncBurs (ViSession instrSession,
                    ViBoolean DCOffset, ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit, ViReal64 frequency,
                    ViInt32 freqUnit, ViReal64 onTime,
                    ViInt32 onTimeUnit, ViReal64 level,
                    ViInt32 levelUnit, ViReal64 burstOnDelay,
                    ViInt32 delayUnit, ViReal64 lowLevel,
                    ViInt32 lowLevelUnit, ViReal64 interval,
                    ViInt32 intervalUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 17))
        return VI_ERROR_PARAMETER4;
    if (DCOffsetUnit == 12)
        if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
            return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
        return VI_ERROR_PARAMETER6;
    if (freqUnit == 0)
        if (rsupl_invalidViReal64Range (frequency, 10.0, 110.0E3))
            return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (onTimeUnit, 0, 5))
        return VI_ERROR_PARAMETER8;
    if (onTimeUnit == 0)
        if (rsupl_invalidViReal64Range (onTime, 0.0, 60.0))
            return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (levelUnit, 0, 20))
        return VI_ERROR_INSTR_PARAMETER10;
    if (levelUnit == 0) {
        if (rsupl_invalidViReal64Range (level, 0.0, 20.0))
            return VI_ERROR_INSTR_PARAMETER9;
    } else if (levelUnit == 15) {
        if (rsupl_invalidViReal64Range (level, 0.0, 1.0))
            return VI_ERROR_INSTR_PARAMETER9;
    }
    if (rsupl_invalidViInt32Range (delayUnit, 0, 2))
        if (rsupl_invalidViInt32Range (delayUnit, 6, 6))
            return VI_ERROR_INSTR_PARAMETER12;
    if (delayUnit == 0)             
        if (rsupl_invalidViReal64Range (burstOnDelay, 0.0, 60.0))
            return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViInt32Range (lowLevelUnit, 0, 22))
        return VI_ERROR_INSTR_PARAMETER14;
    if (lowLevelUnit == 18) {
        if (rsupl_invalidViReal64Range (lowLevel, 0.0, 1.0))
            return VI_ERROR_INSTR_PARAMETER13;
    } else if (lowLevelUnit == 0) {
        if (rsupl_invalidViReal64Range (lowLevel, 0.0, 20.0))
            return VI_ERROR_INSTR_PARAMETER13;
    }
    if (rsupl_invalidViInt32Range (intervalUnit, 0, 2))
        if (rsupl_invalidViInt32Range (intervalUnit, 6, 6))
            return VI_ERROR_INSTR_PARAMETER16;
    if (intervalUnit == 0)
        if (rsupl_invalidViReal64Range (interval, 0.0, 60.0))
            return VI_ERROR_INSTR_PARAMETER15;
    
    Fmt (buffer, "%s<:SOUR:FUNC:SHAP BURS;:SOUR:VOLT:OFFS:STAT %s;"
                ":SOUR:FREQ %f%s;:SOUR:ONT %f%s;INT %f%s;ONT:DEL %f%s;"
                ":SOUR:VOLT %f%s;:SOUR:VOLT:LOWL %f%s",
        stateArr[DCOffset], frequency, freqUnit[unitFreqArr],
        onTime, unitTimeArr[onTimeUnit], interval, unitTimeArr[intervalUnit],
        burstOnDelay, unitTimeArr[delayUnit], level, unitLevelArr[levelUnit], 
        lowLevel, unitBrsLowLevelArr[lowLevelUnit]);
    bytesFmt = NumFmtdBytes ();

    if (DCOffset == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }           
        
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Sine2 Burst                                                     */
/* Purpose:  This function configures the sine burst function generator. A   */
/*           sinewave periodically varying between high and low levels is    */
/*           output by the instrument.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncBurs2 (ViSession instrSession,
                    ViBoolean DCOffset,
                    ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit, ViReal64 frequency,
                    ViInt32 freqUnit, ViReal64 onTime,
                    ViInt32 onTimeUnit, ViReal64 level,
                    ViInt32 levelUnit, ViReal64 burstOnDelay,
                    ViInt32 delayUnit, ViReal64 interval,
                    ViInt32 intervalUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 17))
        return VI_ERROR_PARAMETER4;
    if (DCOffsetUnit == 0) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
            return VI_ERROR_PARAMETER3;
    } else if (DCOffsetUnit == 12) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
            return VI_ERROR_PARAMETER3;
    }
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
        return VI_ERROR_PARAMETER6;
    if (freqUnit == 0)
        if (rsupl_invalidViReal64Range (frequency, 10.0, 110.0E3))
            return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (onTimeUnit, 0, 5))
        return VI_ERROR_PARAMETER8;
    if (onTimeUnit == 0)
        if (rsupl_invalidViReal64Range (onTime, 0.0, 60.0))
            return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (levelUnit, 0, 20))
        return VI_ERROR_INSTR_PARAMETER10;
    if (levelUnit == 0) {
        if (rsupl_invalidViReal64Range (level, 0.0, 20.0))
            return VI_ERROR_INSTR_PARAMETER9;
    } else if (levelUnit == 15) {
        if (rsupl_invalidViReal64Range (level, 0.0, 1.0))
            return VI_ERROR_INSTR_PARAMETER9;
    }
    if (rsupl_invalidViInt32Range (delayUnit, 0, 2))
        if (rsupl_invalidViInt32Range (delayUnit, 6, 6))
            return VI_ERROR_INSTR_PARAMETER12;
    if (delayUnit == 0)             
        if (rsupl_invalidViReal64Range (burstOnDelay, 0.0, 60.0))
            return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViInt32Range (intervalUnit, 0, 2))
        if (rsupl_invalidViInt32Range (intervalUnit, 6, 6))
            return VI_ERROR_INSTR_PARAMETER14;
    if (intervalUnit == 0)
        if (rsupl_invalidViReal64Range (interval, 0.0, 60.0))
            return VI_ERROR_INSTR_PARAMETER13;

    Fmt (buffer, "%s<:SOUR:FUNC:SHAP S2P;:SOUR:VOLT:OFFS:STAT %s;"
                ":SOUR:FREQ %f%s;:SOUR:ONT %f%s;INT %f%s;ONT:DEL %f%s;:SOUR:VOLT %f%s",
        stateArr[DCOffset], frequency, freqUnit[unitFreqArr],
        onTime, unitTimeArr[onTimeUnit], interval, unitTimeArr[intervalUnit],
        burstOnDelay, unitTimeArr[delayUnit], level, unitLevelArr[levelUnit]);
    bytesFmt = NumFmtdBytes ();

    if (DCOffset == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }           
        
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Modulation Distortion                                           */
/* Purpose:  This function configures the generator to output a modulation   */
/*           distortion signal.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncModDist (ViSession instrSession,
                    ViBoolean DCOffset,
                    ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit,
                    ViReal64 totalLevel, ViInt32 levelUnit,
                    ViReal64 levelLFUF,
                    ViBoolean freqOffset,
                    ViReal64 lowerFrequency,
                    ViReal64 upperFrequency,
                    ViInt32 freqUnit, ViBoolean dither,
                    ViReal64 ditherValue,
                    ViInt32 ditherUnit, ViInt32 PDF)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViByte buffer[BUFFER_SIZE];
    ViBoolean instrType, mode;

    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 17))
        return VI_ERROR_PARAMETER4;
    if (DCOffsetUnit == 0) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
            return VI_ERROR_PARAMETER3;
    } else if (DCOffsetUnit == 12) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
            return VI_ERROR_PARAMETER3;
    }
    if (rsupl_invalidViInt32Range (levelUnit, 0, 20))
        return VI_ERROR_PARAMETER6;
    if (levelUnit == 0) {
        if (rsupl_invalidViReal64Range (totalLevel, 0.0, 20.0))
            return VI_ERROR_PARAMETER5;
    } else if (levelUnit == 15) {
        if (rsupl_invalidViReal64Range (totalLevel, 0.0, 1.0))
            return VI_ERROR_PARAMETER5;
    }
    
    if (rsupl_invalidViReal64Range (levelLFUF, 1.0, 10.0))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViBooleanRange (freqOffset))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
        return VI_ERROR_INSTR_PARAMETER11;
    if (freqUnit == 0) {
        if (rsupl_invalidViReal64Range (upperFrequency, 4.0E3, 110.0E3))
            return VI_ERROR_INSTR_PARAMETER10;
        if (rsupl_invalidViReal64Range (lowerFrequency, 30.0, 500.0))
            return VI_ERROR_INSTR_PARAMETER9;
    }
    if (rsupl_invalidViBooleanRange (dither))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (ditherUnit, 14, 20))
        return VI_ERROR_INSTR_PARAMETER14;
    if (ditherUnit == 0)
        if (rsupl_invalidViReal64Range (ditherValue, 0.0, 1.0))
            return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViInt32Range (PDF, 0, 2))
        return VI_ERROR_INSTR_PARAMETER15;
    
    Fmt (buffer, "%s<:SOUR:FUNC:SHAP MDIS;:SOUR:VOLT:OFFS:STAT %s;"
                ":SOUR:FREQ:OFFS:STAT %s;:SOUR:VOLT:TOT %f%s;"
                ":SOUR:FREQ1 %f%s;FREQ2 %f%s;VOLT:RAT %f",
        stateArr[DCOffset], stateArr[freqOffset], 
        totalLevel, unitLevelArr[levelUnit], upperFrequency, unitFreqArr[freqUnit], 
        lowerFrequency, unitFreqArr[freqUnit], levelLFUF);
    bytesFmt = NumFmtdBytes ();

    if (DCOffset == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }           
    
    if ((rsupl_status = rsupl_getGenMode (instrSession, &instrType, &mode)) < 0)
            return rsupl_status;
    if (instrType == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:SIN:DITH:STAT %s", stateArr[dither]);
        bytesFmt += NumFmtdBytes ();
        if (dither == VI_TRUE) {
            Fmt (buffer, "%s[a]<;:SOUR:SIN:DITH %f%s;:SOUR:RAND:PDF %s", 
                ditherValue, unitLevelArr[ditherUnit], PDFArr[PDF]);
            bytesFmt += NumFmtdBytes ();
        }
    }   
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Difference Frequency Distortion                                 */
/* Purpose:  This function configures the generator to output a frequency    */
/*           distortion signal.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncDFD (ViSession instrSession,
                    ViBoolean DCOffset, ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit, ViReal64 totalLevel,
                    ViInt32 levelUnit, ViBoolean freqOffset,
                    ViReal64 meanUpperFrequency,
                    ViReal64 differenceFrequency,
                    ViInt32 freqUnit, ViBoolean dither,
                    ViReal64 ditherValue, ViInt32 ditherUnit,
                    ViInt32 PDF, ViBoolean genMode,
                    ViBoolean equalizer,
                    ViString fileNameEqualizer)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType, mode;

    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if ((rsupl_status = rsupl_getGenMode (instrSession, &instrType, &mode)) < 0)
        return rsupl_status;
    if (instrType == VI_TRUE) {
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 11, 17))
            return VI_ERROR_PARAMETER4;
        if (DCOffsetUnit == 12)
            if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
                return VI_ERROR_PARAMETER3;
        if (rsupl_invalidViInt32Range (levelUnit, 14, 20))
            return VI_ERROR_PARAMETER6;
        if (levelUnit == 15)
            if (rsupl_invalidViReal64Range (totalLevel, 0.0, 1.0))
                return VI_ERROR_PARAMETER5;
    } else {
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 11))
            return VI_ERROR_PARAMETER4;
        if (DCOffsetUnit == 0)
            if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
                return VI_ERROR_PARAMETER3;
        if (rsupl_invalidViInt32Range (levelUnit, 0, 14))
            return VI_ERROR_PARAMETER6;
        if (levelUnit == 0)
            if (rsupl_invalidViReal64Range (totalLevel, 0.0, 20.0))
                return VI_ERROR_PARAMETER5;
    }           
    if (rsupl_invalidViBooleanRange (freqOffset))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
        return VI_ERROR_INSTR_PARAMETER10;
    if (freqUnit == 0) {
        if (rsupl_invalidViReal64Range (meanUpperFrequency, 200.0, 109.5E3))
            return VI_ERROR_PARAMETER8;
        if (rsupl_invalidViReal64Range (differenceFrequency, 80.0, 2.0E3))
            return VI_ERROR_INSTR_PARAMETER9;
    }
    if (rsupl_invalidViBooleanRange (dither))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViInt32Range (ditherUnit, 14, 20))
        return VI_ERROR_INSTR_PARAMETER13;
    if (ditherUnit == 0)
        if (rsupl_invalidViReal64Range (ditherValue, 0.0, 1.0))
            return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (PDF, 0, 2))
        return VI_ERROR_INSTR_PARAMETER14;
    if (rsupl_invalidViBooleanRange (genMode))
        return VI_ERROR_INSTR_PARAMETER15;
    if (rsupl_invalidViBooleanRange (equalizer))
        return VI_ERROR_INSTR_PARAMETER16;
    
    Fmt (buffer, "%s<:SOUR:FUNC:SHAP DFD;MODE %s;:SOUR:VOLT:OFFS:STAT %s;"
                ":SOUR:FREQ:OFFS:STAT %s;:SOUR:VOLT:TOT %f%s;:SOUR:FREQ:DIFF %f%s;"
                ":SOUR:VOLT:EQU:STAT %s",
        dfdGenModeArr[genMode], stateArr[DCOffset], stateArr[freqOffset], 
        totalLevel, unitLevelArr[levelUnit], differenceFrequency, unitFreqArr[freqUnit], stateArr[equalizer]);
    bytesFmt = NumFmtdBytes ();

    if (DCOffset == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }           
    if (equalizer == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST EQU, \"%s\"", fileNameEqualizer);
        bytesFmt += NumFmtdBytes ();
    }
    if (genMode == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:FREQ:MEAN %f%s", meanUpperFrequency, unitFreqArr[freqUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    else {
        Fmt (buffer, "%s[a]<;:SOUR:FREQ %f%s", meanUpperFrequency, unitFreqArr[freqUnit]);
        bytesFmt += NumFmtdBytes ();
    }

    if (instrType == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:SIN:DITH:STAT %s", stateArr[dither]);
        bytesFmt += NumFmtdBytes ();
        if (dither == VI_TRUE) {
            Fmt (buffer, "%s[a]<;:SOUR:SIN:DITH %f%s;:SOUR:RAND:PDF %s", 
                ditherValue, unitLevelArr[ditherUnit], PDFArr[PDF]);
            bytesFmt += NumFmtdBytes ();
        }
    }   
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Random                                                          */
/* Purpose:  This function configures the generator to output a random       */
/*           signal.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncRnd (ViSession instrSession,
                    ViBoolean DCOffset, ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit, ViBoolean level,
                    ViReal64 levelValue, ViInt32 levelUnit,
                    ViBoolean spacing, ViReal64 spacingValueHz,
                    ViReal64 lowerFreqWhiteHz,
                    ViReal64 upperFreqWhiteHz,
                    ViReal64 meanFreq13OctHz, ViInt32 shape,
                    ViString fileNameShape, ViInt32 PDF,
                    ViBoolean domain, ViBoolean equalizer,
                    ViString fileNameEqualizer)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType, mode;
    
    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (level))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (levelUnit, 0, 23))
        return VI_ERROR_PARAMETER7;
    if ((rsupl_status = rsupl_getGenMode (instrSession, &instrType, &mode)) < 0)
        return rsupl_status;
    if (instrType == VI_TRUE)
        if (level == VI_TRUE)
            return VI_ERROR_PARAMETER5;
    if (instrType == VI_TRUE) {
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 11, 17))
            return VI_ERROR_PARAMETER4;
        if (DCOffsetUnit == 12)
            if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
                return VI_ERROR_PARAMETER3;
        if (levelUnit == 11)
            if (rsupl_invalidViReal64Range (levelValue, 0.0, 1.0))
                return VI_ERROR_PARAMETER6;
    }           
    else {
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 11))
            return VI_ERROR_PARAMETER4;
        if (DCOffsetUnit == 0)
            if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
                return VI_ERROR_PARAMETER3;
        if (rsupl_invalidViInt32Range (levelUnit, 0, 11))
            return VI_ERROR_PARAMETER7;
        if (levelUnit == 0)
            if (rsupl_invalidViReal64Range (levelValue, 0.0, 20.0))
                return VI_ERROR_PARAMETER6;
    }           
    if (rsupl_invalidViBooleanRange (spacing))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViReal64Range (spacingValueHz, 0.0, 110.0E3))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViReal64Range (lowerFreqWhiteHz, spacingValueHz, 110.0E3))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViReal64Range (upperFreqWhiteHz, lowerFreqWhiteHz + spacingValueHz, 110.0E3))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViReal64Range (meanFreq13OctHz, 0.0, 110.0E3))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (shape, 0, 3))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViInt32Range (PDF, 0, 2))
        return VI_ERROR_INSTR_PARAMETER15;
    if (rsupl_invalidViBooleanRange (domain))
        return VI_ERROR_INSTR_PARAMETER16;
    if (rsupl_invalidViBooleanRange (equalizer))
        return VI_ERROR_INSTR_PARAMETER17;
    
    Fmt (buffer, "%s<:SOUR:FUNC:SHAP RAND");
    bytesFmt = NumFmtdBytes ();
    
    if (((instrType == VI_TRUE) && (mode == VI_TRUE)) || (instrType == VI_FALSE)) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS:STAT %s", stateArr[DCOffset]);
        bytesFmt += NumFmtdBytes ();
        if (DCOffset == VI_TRUE) {
            Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
            bytesFmt += NumFmtdBytes ();        
        }           
    }
    Fmt (buffer, "%s[a]<;:SOUR:VOLT:TOT:%s %f%s", levelTypeArr[level], levelValue, unitJittRndLevelArr[levelUnit]);
    bytesFmt += NumFmtdBytes ();
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if (domain == VI_TRUE) {
        Fmt (buffer, "%s<:SOUR:RAND:DOM FREQ;SPAC:MODE %s;:SOUR:VOLT:EQU:STAT %s",
            rndSpacingArr[spacing], stateArr[equalizer]);
        bytesFmt = NumFmtdBytes ();
        if (equalizer == VI_TRUE) {
            Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST EQU, \"%s\"", fileNameEqualizer);
            bytesFmt += NumFmtdBytes ();
        }
        if (spacing == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SOUR:RAND:SPAC:FREQ %f", spacingValueHz);
            bytesFmt += NumFmtdBytes ();
        }
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        

        switch (shape) {
            case 0:
                Fmt (buffer, "%s<:SOUR:RAND:SHAP WHIT;:SOUR:RAND:FREQ:LOW %f;UPP %f",
                    lowerFreqWhiteHz, upperFreqWhiteHz);
                break;
            case 1:
                Fmt (buffer, "%s<:SOUR:RAND:SHAP PINK;:SOUR:RAND:FREQ:LOW %f;UPP %f",
                    lowerFreqWhiteHz, upperFreqWhiteHz);
                break;
            case 2:
                Fmt (buffer, "%s<:SOUR:RAND:SHAP TOCT;:SOUR:FREQ:MEAN %f", 
                meanFreq13OctHz);
                break;
            default:
                Fmt (buffer, "%s<:SOUR:RAND:SHAP ARB;:MMEM:LOAD:LIST ARB, \"%s\"", 
                fileNameShape);
                break;
        }
        bytesFmt = NumFmtdBytes ();
    }       
    else {
        Fmt (buffer, "%s<:SOUR:RAND:DOM TIME;PDF %s", PDFArr[PDF]);
        bytesFmt = NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Arbitrary                                                       */
/* Purpose:  This function configures the generator to output a user defined */
/*           arbitrary signal.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncArb (ViSession instrSession,
                    ViBoolean DCOffset, ViReal64 DCOffsetValue,
                    ViInt32 DCOffsetUnit, ViBoolean levelType,
                    ViReal64 levelValue, ViInt32 levelUnit,
                    ViString fileName)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType, mode;
    
    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (levelType))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (levelUnit, 0, 23))
        return VI_ERROR_PARAMETER7;
    
    if ((rsupl_status = rsupl_getGenMode (instrSession, &instrType, &mode)) < 0)
        return rsupl_status;

    if (instrType == VI_TRUE) {
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 11, 17))
            return VI_ERROR_PARAMETER4;
        if (DCOffsetUnit == 12)
            if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
                return VI_ERROR_PARAMETER3;
        if (levelUnit == 11)
            if (rsupl_invalidViReal64Range (levelValue, 0.0, 1.0))
                return VI_ERROR_PARAMETER6;
    } else {
        if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 11))
            return VI_ERROR_PARAMETER4;
        if (DCOffsetUnit == 0)
            if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
                return VI_ERROR_PARAMETER3;
        if (rsupl_invalidViInt32Range (levelUnit, 0, 11))
            return VI_ERROR_PARAMETER7;
        if (levelUnit == 0)
            if (rsupl_invalidViReal64Range (levelValue, 0.0, 20.0))
                return VI_ERROR_PARAMETER6;
    }           

    Fmt (buffer, "%s<:SOUR:FUNC:SHAP USER;:SOUR:VOLT:TOT:%s %f%s", 
            levelTypeArr[levelType], levelValue, 
            unitJittRndLevelArr[levelUnit]);
    bytesFmt = NumFmtdBytes ();
    if (((instrType == VI_TRUE) && (mode == VI_TRUE)) || (instrType == VI_FALSE)) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS:STAT %s", stateArr[DCOffset]);
        bytesFmt += NumFmtdBytes ();
        if (DCOffset == VI_TRUE) {
            Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
            bytesFmt += NumFmtdBytes ();        
        }           
    }
    Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST ARB, \"%s\"", fileName);
        bytesFmt += NumFmtdBytes ();
        
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}        

/*===========================================================================*/
/* Function: Polarity Test Signal                                            */
/* Purpose:  This function configures the the polarity test signal           */
/*           generation.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenFuncPol (ViSession instrSession,
                    ViBoolean DCOffset, ViReal64 DCOffsetValue,
                    ViReal64 levelValue, ViInt32 DCOffsetUnit,
                    ViInt32 levelUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViBooleanRange (DCOffset))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (DCOffsetUnit, 0, 17))
        return VI_ERROR_PARAMETER4;
    if (DCOffsetUnit == 0) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -10.0, 10.0))
            return VI_ERROR_PARAMETER3;
    } else if (DCOffsetUnit == 12) {
        if (rsupl_invalidViReal64Range (DCOffsetValue, -1.0, 1.0))
            return VI_ERROR_PARAMETER3;
    }
    if (rsupl_invalidViInt32Range (levelUnit, 0, 20))
        return VI_ERROR_PARAMETER6;
    if (levelUnit == 0) {
        if (rsupl_invalidViReal64Range (levelValue, 0.0, 20.0))
            return VI_ERROR_PARAMETER5;
    } else if (levelUnit == 15) {
        if (rsupl_invalidViReal64Range (levelValue, 0.0, 1.0))
            return VI_ERROR_PARAMETER5;
    }

    Fmt (buffer, "%s<:SOUR:FUNC:SHAP POL;:SOUR:VOLT:OFFS:STAT %s;:SOUR:VOLT %f%s", 
        stateArr[DCOffset], levelValue, unitLevelArr[levelUnit]);
    bytesFmt = NumFmtdBytes ();
    if (DCOffset == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SOUR:VOLT:OFFS %f%s", DCOffsetValue, unitDCOffsArr[DCOffsetUnit]);
        bytesFmt += NumFmtdBytes ();
    }           
        
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}        

/*===========================================================================*/
/* Function: Sweep Config                                                    */
/* Purpose:  This function configures the sweep system of the audio          */
/*           generator.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenSweConf (ViSession instrSession, ViInt32 sweCtrl,
    ViInt32 nextStep, ViReal64 dwellTime, ViString fileDwell, ViInt32 xAxis,
    ViInt32 zAxis, ViString fileX, ViString fileZ)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (sweCtrl, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (nextStep, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (dwellTime, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (xAxis, 0, 3))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (zAxis, 0, 4))
        return VI_ERROR_PARAMETER7;

    if (sweCtrl == 0) {
        Fmt (buffer, "%s<:SOUR:FREQ:MODE %s", sweCtrlArr[sweCtrl]);
        bytesFmt = NumFmtdBytes ();
    }
    else {  
        Fmt (buffer, "%s<:SOUR:%s:MODE %s1", sweFuncArr[xAxis], sweCtrlArr[sweCtrl]);
        bytesFmt = NumFmtdBytes ();
        if (zAxis != 4) {
            Fmt (buffer, "%s[a]<;:SOUR:%s:MODE %s2", sweFuncArr[zAxis], sweCtrlArr[sweCtrl]);
            bytesFmt += NumFmtdBytes ();
        }
        else {
            Fmt (buffer, "%s[a]<;:SOUR:OFF:MODE %s2", sweCtrlArr[sweCtrl]);
            bytesFmt += NumFmtdBytes ();
        }
        Fmt (buffer, "%s[a]<;:SOUR:SWE:MODE %s", sweOperArr[sweCtrl]);
        bytesFmt += NumFmtdBytes ();
        if (sweCtrl < 3) {
            switch (nextStep) {
                case 1:
                    Fmt (buffer, "%s[a]<;:SOUR:SWE:NEXT DWEL;DWEL %fs", dwellTime);
                    break;
                case 2:
                    Fmt (buffer, "%s[a]<;:SOUR:SWE:NEXT LIST;:MMEM:LOAD:LIST DWEL, \"%s\"", fileDwell);
                    break;
                default:
                    Fmt (buffer, "%s[a]<;:SOUR:SWE:NEXT ASYN");
                    break;
            }
            bytesFmt += NumFmtdBytes ();
        }           
        if (sweCtrl == 2 || sweCtrl == 4) {
            Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST %s, \"%s\"", sweFuncArr[xAxis], fileX);
            bytesFmt += NumFmtdBytes ();
            if (zAxis != 4) {
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST %s, \"%s\"", sweFuncArr[zAxis], fileZ);
                bytesFmt += NumFmtdBytes ();
            }   
        }
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Frequency Sweep                                                 */
/* Purpose:  This function configures frequency sweep parameters.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenSweFreq (ViSession instrSession, ViReal64 startFreq,
    ViReal64 stopFreq, ViReal64 ptsStep, ViInt32 spacing, ViBoolean axis, ViInt32 freqUnit, 
    ViInt32 stepUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (freqUnit, 0, 8))
        return VI_ERROR_PARAMETER7;
    if (freqUnit == 0) {
        if (rsupl_invalidViReal64Range (startFreq, 2.0, 110.0E3))
            return VI_ERROR_PARAMETER2;
        if (rsupl_invalidViReal64Range (stopFreq, 2.0, 110.0E3))
            return VI_ERROR_PARAMETER3;
    }           
    if (rsupl_invalidViInt32Range (spacing, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (stepUnit, 0, 8))
        return VI_ERROR_PARAMETER8;
    switch (spacing) {
        case 2:
            if (stepUnit == 0)
                if (rsupl_invalidViReal64Range (ptsStep, 2.0, 110.0E3))
                    return VI_ERROR_PARAMETER4;
            break;
        case 3:
            if (rsupl_invalidViReal64Range (ptsStep, 0.001, 1.0E3))
                return VI_ERROR_PARAMETER4;
            break;
        default:
            if (rsupl_invalidViReal64Range (ptsStep, 2.0, 1024.0))
                return VI_ERROR_PARAMETER4;
            break;
    }
    if (rsupl_invalidViBooleanRange (axis))
        return VI_ERROR_PARAMETER6;

    Fmt (buffer, "%s<:SOUR:FREQ:MODE SWE%d[b4];STAR %f%s;STOP %f%s;:SOUR:SWE:FREQ:SPAC %s", 
        axis + 1, startFreq, unitFreqArr[freqUnit], stopFreq, unitFreqArr[freqUnit], 
        sweSpacingArr[spacing]);
    bytesFmt = NumFmtdBytes ();
    if (spacing < 2) {
        Fmt (buffer, "%s[a]< %d[b4]", (int)ptsStep);
        bytesFmt += NumFmtdBytes ();
    }       
    else {
        Fmt (buffer, "%s[a]< %f", ptsStep);
        bytesFmt += NumFmtdBytes ();
        if (spacing == 2) {
            Fmt (buffer, "%s[a]<%s", unitFreqArr[stepUnit]);
            bytesFmt += NumFmtdBytes ();
        }
    }           
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Level Sweep                                                     */
/* Purpose:  This function configures level sweep parameters.                */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenSweLev (ViSession instrSession, ViReal64 startLevel,
    ViReal64 stopLevel, ViReal64 ptsStep, ViInt32 spacing, ViBoolean axis, 
    ViInt32 levelUnit, ViInt32 stepUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (levelUnit, 0, 26))
            return VI_ERROR_PARAMETER7;
    if (levelUnit == 0) {
        if (rsupl_invalidViReal64Range (startLevel, 0.0, 12.0))
            return VI_ERROR_PARAMETER2;
        if (rsupl_invalidViReal64Range (stopLevel, 0.0, 12.0))
            return VI_ERROR_PARAMETER3;
    }           
    if (rsupl_invalidViInt32Range (spacing, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (stepUnit, 0, 26))
        return VI_ERROR_PARAMETER8;
    switch (spacing) {
        case 2:
            if (stepUnit == 0)
                if (rsupl_invalidViReal64Range (ptsStep, 0.0, 12.0))
                    return VI_ERROR_PARAMETER4;
            break;
        case 3:
            if (rsupl_invalidViReal64Range (ptsStep, 0.001, 1.0E3))
                return VI_ERROR_PARAMETER4;
            break;
        default:
            if (rsupl_invalidViReal64Range (ptsStep, 2.0, 1024.0))
                return VI_ERROR_PARAMETER4;
            break;
    }
    if (rsupl_invalidViBooleanRange (axis))
        return VI_ERROR_PARAMETER6;

    Fmt (buffer, "%s<:SOUR:VOLT:MODE SWE%d[b4];STAR %f%s;STOP %f%s;:SOUR:SWE:VOLT:SPAC %s", 
        axis + 1, startLevel, unitLevelArr[levelUnit], stopLevel, unitLevelArr[levelUnit], 
        sweSpacingArr[spacing]);
    bytesFmt = NumFmtdBytes ();
    if (spacing < 2) {
        Fmt (buffer, "%s[a]< %d[b4]", (int)ptsStep);
        bytesFmt += NumFmtdBytes ();
    }       
    else {
        Fmt (buffer, "%s[a]< %f", ptsStep);
        bytesFmt += NumFmtdBytes ();
        if (spacing == 2) {
            Fmt (buffer, "%s[a]<%s", unitLevelArr[stepUnit]);
            bytesFmt += NumFmtdBytes ();
        }
    }           
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: On-Time Sweep                                                   */
/* Purpose:  This function configures on-time sweep parameters.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenSweOnTm (ViSession instrSession, ViReal64 startOnTime,
    ViReal64 stopOnTime, ViReal64 ptsStep, ViInt32 spacing, ViBoolean axis, 
    ViInt32 onTimeUnit, ViInt32 stepUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (onTimeUnit, 0, 5))
            return VI_ERROR_PARAMETER7;
    if (onTimeUnit == 0) {
        if (rsupl_invalidViReal64Range (startOnTime, 0.0, 60.0))
            return VI_ERROR_PARAMETER2;
        if (rsupl_invalidViReal64Range (stopOnTime, 0.0, 60.0))
            return VI_ERROR_PARAMETER3;
    }           
    if (rsupl_invalidViInt32Range (spacing, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (stepUnit, 0, 5))
        return VI_ERROR_PARAMETER8;
    switch (spacing) {
        case 2:
            if (stepUnit == 0)
                if (rsupl_invalidViReal64Range (ptsStep, 0.0, 60.0))
                    return VI_ERROR_PARAMETER4;
            break;
        case 3:
            if (rsupl_invalidViReal64Range (ptsStep, 0.001, 1.0E3))
                return VI_ERROR_PARAMETER4;
            break;
        default:
            if (rsupl_invalidViReal64Range (ptsStep, 2.0, 1024.0))
                return VI_ERROR_PARAMETER4;
            break;
    }
    if (rsupl_invalidViBooleanRange (axis))
        return VI_ERROR_PARAMETER6;

    Fmt (buffer, "%s<:SOUR:ONT:MODE SWE%d[b4];STAR %f%s;STOP %f%s;:SOUR:SWE:ONT:SPAC %s", 
        axis + 1, startOnTime, unitTimeArr[onTimeUnit], stopOnTime, unitTimeArr[onTimeUnit], 
        sweSpacingArr[spacing]);
    bytesFmt = NumFmtdBytes ();
    if (spacing < 2) {
        Fmt (buffer, "%s[a]< %d[b4]", (int)ptsStep);
        bytesFmt += NumFmtdBytes ();
    }       
    else {
        Fmt (buffer, "%s[a]< %f", ptsStep);
        bytesFmt += NumFmtdBytes ();
        if (spacing == 2) {
            Fmt (buffer, "%s[a]<%s", unitTimeArr[stepUnit]);
            bytesFmt += NumFmtdBytes ();
        }
    }           
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Interval Sweep                                                  */
/* Purpose:  This function configures interval sweep parameters.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confGenSweInt (ViSession instrSession, 
            ViReal64 startInterval, ViReal64 stopInterval, ViReal64 ptsStep, 
            ViInt32 spacing, ViBoolean axis, ViInt32 intervalUnit, 
    ViInt32 stepUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (intervalUnit, 0, 3))
            return VI_ERROR_PARAMETER7;
    if (intervalUnit == 0) {
        if (rsupl_invalidViReal64Range (startInterval, 0.0, 60.0))
            return VI_ERROR_PARAMETER2;
        if (rsupl_invalidViReal64Range (stopInterval, 0.0, 60.0))
            return VI_ERROR_PARAMETER3;
    }           
    if (rsupl_invalidViInt32Range (spacing, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (stepUnit, 0, 3))
        return VI_ERROR_PARAMETER8;
    switch (spacing) {
        case 2:
            if (stepUnit == 0)
                if (rsupl_invalidViReal64Range (ptsStep, 0.0, 60.0))
                    return VI_ERROR_PARAMETER4;
            break;
        case 3:
            if (rsupl_invalidViReal64Range (ptsStep, 0.001, 1.0E3))
                return VI_ERROR_PARAMETER4;
            break;
        default:
            if (rsupl_invalidViReal64Range (ptsStep, 2.0, 1024.0))
                return VI_ERROR_PARAMETER4;
            break;
    }
    if (rsupl_invalidViBooleanRange (axis))
        return VI_ERROR_PARAMETER6;

    Fmt (buffer, "%s<:SOUR:INT:MODE SWE%d[b4];STAR %f%s;STOP %f%s;:SOUR:SWE:INT:SPAC %s", 
        axis + 1, startInterval, unitIntervalArr[intervalUnit], stopInterval, 
        unitIntervalArr[intervalUnit], sweSpacingArr[spacing]);
    bytesFmt = NumFmtdBytes ();
    if (spacing < 2) {
        Fmt (buffer, "%s[a]< %d[b4]", (int)ptsStep);
        bytesFmt += NumFmtdBytes ();
    }       
    else {
        Fmt (buffer, "%s[a]< %f", ptsStep);
        bytesFmt += NumFmtdBytes ();
        if (spacing == 2) {
            Fmt (buffer, "%s[a]<%s", unitIntervalArr[stepUnit]);
            bytesFmt += NumFmtdBytes ();
        }
    }           
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Analog Analyzer Setup                                           */
/* Purpose:  This function configures the analog analyzer.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerAnalStp (ViSession instrSession,
                    ViReal64 refImp, ViInt32 channels,
                    ViBoolean chan1Inp, ViBoolean chan2Inp,
                    ViInt32 chan1Imp, ViInt32 chan2Imp, ViBoolean chan1Com,
                    ViBoolean chan2Com, ViInt32 chan1RangMode,
                    ViInt32 chan2RangMode, ViInt32 chan1Range,
                    ViInt32 chan2Range, ViBoolean instrType)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViReal64Range (refImp, 1.0E-3, 1.0E5))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (channels, 0, 4))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (chan1Inp))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViBooleanRange (chan2Inp))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (chan1Imp, 0, 2))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (chan2Imp, 0, 2))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViBooleanRange (chan1Com))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViBooleanRange (chan2Com))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (chan1RangMode, 0, 2))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (chan2RangMode, 0, 2))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViInt32Range (chan1Range, 0, 13))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (chan2Range, 0, 13))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViBooleanRange (instrType))
        return VI_ERROR_INSTR_PARAMETER14;

    Fmt (buffer, "%s<:INST2:NSEL %s;:SENS:POW:REF:RES %f;:INP:SEL %s", 
        analyzerTypeArr[instrType], refImp, analyzerInputArr[channels]);
    bytesFmt = NumFmtdBytes ();

    switch (channels) {
        case 1:
            Fmt (buffer, "%s[a]<;:INP2:TYPE %s", chan2InpArr[chan2Inp]);
            bytesFmt += NumFmtdBytes ();
            if (chan2Inp == 0) {
                Fmt (buffer, "%s[a]<;IMP %s;LOW %s", chanImpArr[chan2Imp], chanComArr[chan2Com]);
                bytesFmt += NumFmtdBytes ();
            }   
            if (chan2RangMode == 0)
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG2:AUTO ON");
            else
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG2:AUTO OFF;%s %s",
                    chanRangModeArr[chan2RangMode - 1], chanRangeArr[chan2Range]);
            bytesFmt += NumFmtdBytes ();
            break;
        case 2:
            Fmt (buffer, "%s[a]<;:INP1:TYPE %s;:INP2:TYPE %s", chan1InpArr[chan1Inp], chan2InpArr[chan2Inp]);
            bytesFmt += NumFmtdBytes ();
            if (chan1Inp == 0) {
                Fmt (buffer, "%s[a]<;:INP1:IMP %s;LOW %s", chanImpArr[chan1Imp], chanComArr[chan1Com]);
                bytesFmt += NumFmtdBytes ();
            }   
            if (chan2Inp == 0) {
                Fmt (buffer, "%s[a]<;:INP2:IMP %s;LOW %s", chanImpArr[chan2Imp], chanComArr[chan2Com]);
                bytesFmt += NumFmtdBytes ();
            }   
            if (chan1RangMode == 0)
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG1:AUTO ON");
            else
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG1:AUTO OFF;%s %s",
                    chanRangModeArr[chan1RangMode - 1], chanRangeArr[chan1Range]);
            bytesFmt += NumFmtdBytes ();
            if (chan2RangMode == 0)
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG2:AUTO ON");
            else
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG2:AUTO OFF;%s %s",
                    chanRangModeArr[chan2RangMode - 1], chanRangeArr[chan2Range]);
            bytesFmt += NumFmtdBytes ();
            break;
        default:
            Fmt (buffer, "%s[a]<;:INP1:TYPE %s", chan1InpArr[chan1Inp]);
            bytesFmt += NumFmtdBytes ();
            if (chan1Inp == 0) {
                Fmt (buffer, "%s[a]<;IMP %s;LOW %s", chanImpArr[chan1Imp], chanComArr[chan1Com]);
                bytesFmt += NumFmtdBytes ();
            }   
            if (chan1RangMode == 0)
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG1:AUTO ON");
            else
                Fmt (buffer, "%s[a]<;:SENS:VOLT:RANG1:AUTO OFF;%s %s",
                    chanRangModeArr[chan1RangMode - 1], chanRangeArr[chan1Range]);
            bytesFmt += NumFmtdBytes ();
            break;
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Digital Analyzer Setup                                          */
/* Purpose:  This function the digital analyzer.                             */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerDigStp (ViSession instrSession, ViInt32 measMode,
    ViInt32 channels, ViInt32 jitterRef, ViInt32 input, ViBoolean sync,
    ViReal64 samplFreqVal, ViInt32 samplFreq, ViInt32 audioBits)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (measMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (channels, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (jitterRef, 0, 4))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (input, 0, 3))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViBooleanRange (sync))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViReal64Range (samplFreqVal, 100.0, 52.5E3))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (samplFreq, 0, 3))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (audioBits, 8, 24))
        return VI_ERROR_INSTR_PARAMETER9;

    Fmt (buffer, "%s<:INST2:NSEL 4;:SENS:DIG:FEED %s;:INP:TYPE %s", 
        measModeArr[measMode], digInpArr[input]);
    bytesFmt = NumFmtdBytes ();
    
    switch (measMode) {
        case 0:
            Fmt (buffer, "%s[a]<;:INP:SEL %s;AUD %d[b4];SAMP:FREQ:MODE %s;:SENS:DIG:SYNC:SOUR %s",
                digChanArr[channels], audioBits, digSamplFreqArr[samplFreq], 
                digSyncSourArr[sync]);
            bytesFmt += NumFmtdBytes ();
            if (samplFreq == 0) {
                Fmt (buffer, "%s[a]<;:INP:SAMP:FREQ %fHZ", samplFreqVal);
                bytesFmt += NumFmtdBytes ();
            }
            break;
        case 1:
            Fmt (buffer, "%s[a]<;:SENS:DIG:SYNC:REF %s", jitterRefArr[jitterRef]);
            bytesFmt += NumFmtdBytes ();
            break;
    }
        
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Post FFT                                                        */
/* Purpose:  This function enables and configures post FFT subsequent to a   */
/*           specified analyzer function.                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFFT (ViSession instrSession, ViInt32 window,
    ViInt32 size, ViReal64 betaFactor)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (window, 0, 9))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (size, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (betaFactor, 1.0, 20.0))
        return VI_ERROR_PARAMETER4;
        

    Fmt (buffer, "%s<:CALC:TRAN:FREQ:%s", FFTWindowArr[window]);
        bytesFmt = NumFmtdBytes ();
    if (window > 0) {
        Fmt (buffer, "%s[a]<;FFT %s", FFTSizeArr[size]);
        bytesFmt += NumFmtdBytes ();
    }
    if (window == 9) {
        Fmt (buffer, "%s[a]<;WIND:BETA %f", betaFactor);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Settling Config                                                 */
/* Purpose:  This function enables and configures the settling procedure of  */
/*           trigger, function, frequency and phase.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerSettl (ViSession instrSession, ViInt32 settlType,
    ViInt32 samples, ViReal64 tolerance, ViBoolean toleranceUnit, ViReal64 resolution,
    ViInt32 resolutionUnit, ViReal64 timeout, ViInt32 parameter)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (settlType, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (samples, 2, 100))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (toleranceUnit))
        return VI_ERROR_PARAMETER5;
    if (toleranceUnit == 0)
        if (rsupl_invalidViReal64Range (tolerance, 0.001, 10.0))
            return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (resolutionUnit, 0, 18))
        return VI_ERROR_PARAMETER7;
    if (resolutionUnit == 0)
        if (rsupl_invalidViReal64Range (resolution, 0.0, 10.0))
            return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViReal64Range (timeout, 0.001, 2.0E3))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (parameter, 0, 3))
        return VI_ERROR_INSTR_PARAMETER9;

    Fmt (buffer, "%s<:SENS%s:SETT:MODE %s", settlParamArr[parameter], settlTypeArr[settlType]);
        bytesFmt = NumFmtdBytes ();
    if (settlType > 0) {
        Fmt (buffer, "%s[a]<;COUN %d[b4]", samples);
        bytesFmt += NumFmtdBytes ();
        if (settlType < 3) {
            Fmt (buffer, "%s[a]<;RES %f%s", resolution, settlingResUnitArr[resolutionUnit]);
            bytesFmt += NumFmtdBytes ();
        }
        if (settlType < 3) {
            Fmt (buffer, "%s[a]<;TOL %f%s", tolerance, triggerValUnitArr[toleranceUnit]);
            bytesFmt += NumFmtdBytes ();
        }
        if (parameter > 0) {
            Fmt (buffer, "%s[a]<;TOUT %f", timeout);
            bytesFmt += NumFmtdBytes ();
        }
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Analyzer Trigger Config                                         */
/* Purpose:  This function configures the analyzer trigger.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerTrig (ViSession instrSession, ViInt32 startCond,
    ViReal64 delay, ViInt32 points, ViReal64 minLevel, ViInt32 minLevUnit,
    ViReal64 time, ViReal64 variation, ViReal64 startVal, ViReal64 stopVal,
    ViInt32 startStopValUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (startCond, 0, 10))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (delay, 0.0, 10.0))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (points, 2, 1024))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (minLevUnit, 0, 15))
        return VI_ERROR_PARAMETER6;
    if (minLevUnit == 0)
        if (rsupl_invalidViReal64Range (minLevel, 0.0, 1.0E3))
            return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViReal64Range (time, 0.0, 10.0))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViReal64Range (variation, 0.1, 50.0))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (startStopValUnit, 0, 16))
        return VI_ERROR_INSTR_PARAMETER11;
    if (startStopValUnit == 16) {
        if (rsupl_invalidViReal64Range (startVal, 0.0, 125.0E3))
            return VI_ERROR_INSTR_PARAMETER9;
        if (rsupl_invalidViReal64Range (stopVal, 0.0, 125.0E3))
            return VI_ERROR_INSTR_PARAMETER10;
    }               

    switch (startCond) {
        case 0:
            Fmt (buffer, "%s<:TRIG:SOUR IMM;DEL %fS", delay);       
            break;
        case 1:
            Fmt (buffer, "%s<:TRIG:SOUR TIM;TIM %fS;COUN %d[b4]", time, points);
            break;
        case 2:     
        case 3:
            Fmt (buffer, "%s<:TRIG:SOUR CH%d[b4]F;DEL %fS;FREQ:VAR %fPCT;"
                        ":ARM:LEV:MIN %f%s;:ARM:FREQ:STAR %f%s;STOP %f%s", 
                startCond-1, delay, variation, minLevel, 
                settlingResUnitArr[minLevUnit],
                startVal, settlingResUnitArr[startStopValUnit], stopVal, 
                settlingResUnitArr[startStopValUnit]);
            break;
        case 4:     
        case 5:
            Fmt (buffer, "%s<:TRIG:SOUR CH%d[b4]L;DEL %fS;VOLT:VAR %fPCT;"
                        ":ARM:VOLT:STAR %f%s;STOP %f%s", 
                startCond-3, delay, variation, startVal, 
                settlingResUnitArr[startStopValUnit], stopVal, 
                settlingResUnitArr[startStopValUnit]);
            break;
        case 6:     
        case 7:
            Fmt (buffer, "%s<:TRIG:SOUR CH%d[b4]T;DEL %fS;"
                        ":ARM:VOLT:STAR %f%s;STOP %f%s", 
                startCond-5, delay, startVal, settlingResUnitArr[startStopValUnit], 
                stopVal, settlingResUnitArr[startStopValUnit]);
            break;
        case 8:
            Fmt (buffer, "%s<:TRIG:SOUR TCH;TIM %fS;COUN %d[b4]", time, points);            
            break;
        case 9:     
        case 10:
            Fmt (buffer, "%s<:TRIG:SOUR CH%d[b4]R;DEL %fS;FREQ:VAR %fPCT;"
                        ":ARM:LEV:MIN %f%s;:ARM:FREQ:STAR %f%s;STOP %f%s", 
                startCond-8, delay, variation, minLevel, 
                settlingResUnitArr[minLevUnit], startVal, 
                settlingResUnitArr[startStopValUnit], stopVal, 
                settlingResUnitArr[startStopValUnit]);
            break;
    }
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: User Filter Config                                              */
/* Purpose:  This function configures the user-defined filters which can be  */
/*           consequently used in the analyzer functions.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFilterUser (ViSession instrSession, 
                    ViInt32 filterType, ViInt32 filterNumber, ViReal64 passbandFreq, 
                    ViReal64 passbandLowFreq, ViReal64 passbandHighFreq, 
                    ViReal64 passbandCenterFreq, ViReal64 passbandWidthFreq, 
                    ViReal64 attenuation, ViString fileName, ViReal64 delay)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (filterType, 0, 7))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (filterNumber, 0, 8))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (passbandFreq, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViReal64Range (passbandLowFreq, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViReal64Range (passbandHighFreq, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViReal64Range (passbandCenterFreq, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViReal64Range (passbandWidthFreq, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViReal64Range (attenuation, 3.0, 120.0))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViReal64Range (delay, 0.0, 1.0))
        return VI_ERROR_INSTR_PARAMETER11;
    
    switch (filterType) {
        case 0:
        case 1:
            Fmt (buffer, "%s<:SENS:UFIL%d[b4]:%s ON;PASS %fHZ;ATT %fDB", 
                filterNumber+1,  filterTypeArr[filterType], passbandFreq, 
                attenuation);
            break;
        case 2:
        case 3:     
            Fmt (buffer, "%s<:SENS:UFIL%d[b4]:%s ON;ATT %fDB;PASS:LOW %fHZ;UPP %fHZ", 
                filterNumber+1, filterTypeArr[filterType], attenuation, 
                passbandLowFreq, passbandHighFreq);
            break;
        case 4:
            Fmt (buffer, "%s<:SENS:UFIL%d[b4]:%s ON;ATT %fDB;CENT %fHZ;WIDT %fHZ", 
                filterNumber+1, filterTypeArr[filterType], attenuation, 
                passbandCenterFreq, passbandWidthFreq);
        case 5:     
        case 6:     
            Fmt (buffer, "%s<:SENS:UFIL%d[b4]:%s ON;ATT %fDB;CENT %fHZ", 
                filterNumber+1, filterTypeArr[filterType], attenuation, 
                passbandCenterFreq);
            break;
        case 7:     
            Fmt (buffer, "%s<:SENS:UFIL%d[b4]:%s ON;DEL %fS;FILE \"%s\"", 
                filterNumber+1, filterTypeArr[filterType], delay, fileName);
            break;
    }
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Filter Select                                                   */
/* Purpose:  This function selects filters for analyzer functions.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFilterSel (ViSession instrSession, 
                    ViInt32 filter1, ViInt32 filter2, ViInt32 filter3)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (filter1, 0, 24))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (filter2, 0, 24))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (filter3, 0, 24))
        return VI_ERROR_PARAMETER4;
        
    if (filter1 == 0)
        Fmt (buffer, "%s<:SENS:FILT1 OFF");
    else        
        Fmt (buffer, "%s<:SENS:FILT1:%s:STAT ON", filterArr[filter1 - 1]);
    bytesFmt = NumFmtdBytes ();
    if (filter2 == 0)
        Fmt (buffer, "%s[a]<;:SENS:FILT2 OFF");
    else        
        Fmt (buffer, "%s[a]<;:SENS:FILT2:%s:STAT ON", filterArr[filter2 - 1]);
    bytesFmt += NumFmtdBytes ();
    if (filter3 == 0)
        Fmt (buffer, "%s[a]<;:SENS:FILT3 OFF");
    else        
        Fmt (buffer, "%s[a]<;:SENS:FILT3:%s:STAT ON", filterArr[filter3 - 1]);
    bytesFmt += NumFmtdBytes ();
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Limit Check Config                                              */
/* Purpose:  This function selects filter for analyzer functions.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerSingleFilterSel (ViSession instrSession,
                                                     ViInt32 filterNumber,
                                                     ViInt32 filter)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (filterNumber, 1, 3))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (filter, 0, 24))
        return VI_ERROR_PARAMETER3;
        
    if (filter == 0)
        Fmt (buffer, "%s<:SENS:FILT%d OFF", filterNumber);
    else        
        Fmt (buffer, "%s<:SENS:FILT%d:%s:STAT ON", filterNumber, filterArr[filter - 1]);
    bytesFmt = NumFmtdBytes ();
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Limit Check Config                                              */
/* Purpose:  This function enables and configures the limit checking         */
/*           capability of the instrument.                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerLimit (ViSession instrSession, ViInt32 checkMode,
                        ViInt32 checkObject, ViBoolean uppLimit, ViBoolean lowLimit,
                        ViChar _VI_FAR uppFileName[], ViChar _VI_FAR lowFileName[], 
                        ViReal64 uppLimVal, ViReal64 lowLimVal, ViInt32 uppUnit,
                        ViInt32 lowUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (checkMode, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (checkObject, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (uppLimit))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViBooleanRange (lowLimit))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (uppUnit, 0, 29))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (lowUnit, 0, 29))
        return VI_ERROR_INSTR_PARAMETER11;
    if (uppUnit == 0)           
        if (rsupl_invalidViReal64Range (uppLimVal, -1.0E9, 1.0E9))
            return VI_ERROR_PARAMETER8;
    if (lowUnit == 0)           
        if (rsupl_invalidViReal64Range (lowLimVal, -1.0E9, 1.0E9))
            return VI_ERROR_INSTR_PARAMETER9;
        
    switch (checkMode) {
        case 0:
            Fmt (buffer, "%s<:CALC:LIM:UPP:STAT OFF;:CALC:LIM:LOW:STAT OFF");   
            bytesFmt = NumFmtdBytes ();         
            break;
        case 1:
            Fmt (buffer, "%s<:CALC:LIM:LOW:STAT ON;:CALC:LIM:UPP:STAT OFF;"
                        ":CALC:LIM:ON %s", checkArr[checkObject]); 
            bytesFmt = NumFmtdBytes ();         
            if (lowLimit == VI_FALSE)
                Fmt (buffer, "%s[a]<;:CALC:LIM:LOW:VAL %f%s", lowLimVal, limUnitArr[lowUnit]);
            else
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST LIML, \"%s\"", lowFileName);
            bytesFmt += NumFmtdBytes ();
            break;
        case 2:
            Fmt (buffer, "%s<:CALC:LIM:UPP:STAT ON;:CALC:LIM:LOW:STAT OFF;"
                        ":CALC:LIM:ON %s", checkArr[checkObject]);
            bytesFmt = NumFmtdBytes ();         
            if (uppLimit == VI_FALSE)
                Fmt (buffer, "%s[a]<;:CALC:LIM:UPP:VAL %f%s", uppLimVal, limUnitArr[uppUnit]);
            else
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST LIMU, \"%s\"", uppFileName);
            bytesFmt += NumFmtdBytes ();
            break;
        case 3:
            Fmt (buffer, "%s<:CALC:LIM:UPP:STAT ON;:CALC:LIM:LOW:STAT ON;:CALC:LIM:ON %s", checkArr[checkObject]);
            bytesFmt = NumFmtdBytes ();         
            if (uppLimit == VI_FALSE)
                Fmt (buffer, "%s[a]<;:CALC:LIM:UPP:VAL %f%s", uppLimVal, limUnitArr[uppUnit]);
            else
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST LIMU, \"%s\"", uppFileName);
            bytesFmt += NumFmtdBytes ();
            if (lowLimit == VI_FALSE)
                Fmt (buffer, "%s[a]<;:CALC:LIM:LOW:VAL %f%s", lowLimVal, limUnitArr[lowUnit]);
            else
                Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST LIML, \"%s\"", lowFileName);
            bytesFmt += NumFmtdBytes ();
            break;
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Protocol Analysis Config                                        */
/* Purpose:  This function configures protocol analysis.                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerProtocolAnl (ViSession instrSession, 
                    ViInt32 source, ViInt32 format, ViString fileName)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (source, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (format, 0, 3))
        return VI_ERROR_PARAMETER3;
        
    if (source < 2) {
        Fmt (buffer, "%s<:DISP:PROT:SEL %s;:MMEM:LOAD:PAC \"%s\"", protAnlSourceArr[source],
            fileName);
        bytesFmt = NumFmtdBytes ();         
    }
    else {
        Fmt (buffer, "%s<:DISP:PROT:SEL %s;FORM %s", protAnlSourceArr[source], protAnlFormatArr[format]);
        bytesFmt = NumFmtdBytes ();         
        if (format == 3) {
            Fmt (buffer, "%s[a]<;:MMEM:LOAD:PAU \"%s\"", fileName);
            bytesFmt += NumFmtdBytes ();            
        }
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Peak/Phase                                                      */
/* Purpose:  This function configures the peak values measurement and        */
/*           display.                                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerMeasInpDisp (ViSession instrSession, 
                    ViInt32 display, ViInt32 unitCh1, ViInt32 unitCh2, 
                    ViInt32 reference, ViReal64 referenceVal,
    ViInt32 referenceValUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViInt32 chanType = 1;
    
    if (rsupl_invalidViInt32Range (display, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (unitCh1, 0, 28))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (unitCh2, 0, 20))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (referenceValUnit, 0, 21))
        return VI_ERROR_PARAMETER7;
    if (referenceValUnit == 0)
        if (rsupl_invalidViReal64Range (referenceVal, -1.0E3, 1.0E3))
            return VI_ERROR_PARAMETER6;

    if (display == 0) {
        Fmt (buffer, "%s<:SENS2:FUNC \"%s\"", inpDispModeArr[display]);
        if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
            return rsupl_status;        
    }
    else {
        Fmt (buffer, "%s<:SENS2:FUNC \"%s\"", inpDispModeArr[display]);
        bytesFmt = NumFmtdBytes ();
        
        if (display < 3) {
            Fmt (buffer, "%s[a]<;:SENS2:VOLT:REF:MODE %s", rmsRefArr[reference]);
            bytesFmt += NumFmtdBytes ();
            if (reference == 0) {
                Fmt (buffer, "%s[a]<;:SENS2:VOLT:REF %f%s", referenceVal, referenceValUnitArr[referenceValUnit]);
                bytesFmt += NumFmtdBytes ();
            }
        }           
    
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    
        if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
            return rsupl_status;
        
        if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
            return rsupl_status;
        
        switch (chanType) {
            case 1:
                Fmt (buffer, "%s<:SENS2:VOLT:UNIT1 %s", inpDispUnitArr[unitCh1]);               
                break;
            case 2:
                Fmt (buffer, "%s<:SENS2:VOLT:UNIT2 %s", inpDispUnitArr[unitCh2]);
                break;
            default:
                Fmt (buffer, "%s<:SENS2:VOLT:UNIT1 %s;UNIT2 %s", inpDispUnitArr[unitCh1], inpDispUnitArr[unitCh2]);
                break;
        }
        if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
            return rsupl_status;        
    }
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Frequency/Phase/Grp Delay                                       */
/* Purpose:  This function configures the frequency, phase, and group delay  */
/*           measurement and display.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerMeasInpFreq (ViSession instrSession, 
                    ViInt32 display, ViInt32 unitCh1, ViInt32 unitCh2, 
                    ViInt32 freqReference, ViReal64 freqReferenceVal, 
                    ViInt32 phasReference, ViReal64 phasReferenceVal, 
                    ViInt32 phasFormat, ViInt32 unitPhas)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViInt32 chanType = 1;
    
    if (rsupl_invalidViInt32Range (display, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (unitCh1, 0, 8))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (unitCh2, 0, 12))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (freqReference, 0, 6))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViReal64Range (freqReferenceVal, -1.0E6, 1.0E6))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (phasReference, 0, 6))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (unitPhas, 0, 5))
        return VI_ERROR_INSTR_PARAMETER10;
    if (unitPhas == 0)          
        if (rsupl_invalidViReal64Range (phasReferenceVal, -360.0, 360.0))
            return VI_ERROR_PARAMETER8;    
    if (rsupl_invalidViInt32Range (phasFormat, 0, 5))
        return VI_ERROR_INSTR_PARAMETER9;

    if (display == 0) {
        Fmt (buffer, "%s<:SENS3:FUNC \"%s\"", inpDispFreqModeArr[display]);
        if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
            return rsupl_status;        
    }
    else {
        Fmt (buffer, "%s<:SENS3:FUNC \"%s\";:SENS3:FREQ:REF:MODE %s", 
            inpDispFreqModeArr[display], rmsRefArr[freqReference]);
        bytesFmt = NumFmtdBytes ();
        if (freqReference == 0) {
            Fmt (buffer, "%s[a]<;:SENS3:FREQ:REF %fHZ", freqReferenceVal);
            bytesFmt += NumFmtdBytes ();
        }
        
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    
        if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
            return rsupl_status;
        
        if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
            return rsupl_status;
        
        switch (chanType) {
            case 1:
                Fmt (buffer, "%s<:SENS3:FREQ:UNIT1 %s", inpDispUnitFreqArr[unitCh1]);               
                bytesFmt = NumFmtdBytes ();
                break;
            case 2:
                Fmt (buffer, "%s<:SENS3:FREQ:UNIT2 %s", inpDispUnitFreqArr[unitCh2]);
                bytesFmt = NumFmtdBytes ();
                break;
            default:
                Fmt (buffer, "%s<:SENS3:FREQ:UNIT1 %s;:SENS3:PHAS:UNIT2 %s", inpDispUnitFreqArr[unitCh1], 
                    inpDispUnitFreqArr[unitCh2]);
                bytesFmt = NumFmtdBytes ();
                if (display == 2) {
                    Fmt (buffer, "%s[a]<;:SENS3:PHAS:FORM %s;REF:MODE %s", 
                        phasFormatArr[phasFormat], inpPhasRefArr[phasReference]);
                    bytesFmt += NumFmtdBytes ();
                    if (phasReference == 0) {
                        Fmt (buffer, "%s[a]<;:SENS3:PHAS:REF %f%s", phasReferenceVal, 
                            inpDispRefUnitFreqArr[unitPhas]);
                        bytesFmt += NumFmtdBytes ();
                    }
                }                   
                if (display == 3) {
                    Fmt (buffer, "%s[a]<;:SENS3:PHAS:REF:MODE %s", inpPhasRefArr[phasReference]);
                    bytesFmt += NumFmtdBytes ();
                    if (phasReference == 0) {
                        Fmt (buffer, "%s[a]<;:SENS3:PHAS:REF %f%s", phasReferenceVal, 
                            inpDispRefUnitFreqArr[unitPhas]);
                        bytesFmt += NumFmtdBytes ();
                    }
                }                   
                break;
        }
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    }
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Disable Analyzer Function                                       */
/* Purpose:  This function disables analyzer function.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncOff (ViSession instrSession) 
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    Fmt (buffer, "%s<:SENS:FUNC \'OFF\'");
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: RMS & S/N                                                       */
/* Purpose:  This function enables and configures measurement of AC voltages */
/*           of any desired waveform up to 30 V. A superimposed DC is not    */
/*           measured. The signal-to-noise measurement is also available.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncRMSSN (ViSession instrSession,
                    ViBoolean SNMeasure, ViInt32 measTime,
                    ViReal64 measTimeValues, ViInt32 unitChan1,
                    ViInt32 unitChan2, ViReal64 triggerLevel,
                    ViBoolean triggerLevelUnit, ViInt32 reference,
                    ViReal64 referenceValue, ViInt32 referenceValueUnit,
                    ViInt32 notchGain, ViInt32 notchFreq,
                    ViReal64 notchFreqValueHz)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 digMode = 0;
    ViInt32 chanType = 1;
    
    if (rsupl_invalidViBooleanRange (SNMeasure))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (measTime, 0, 4))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (measTimeValues, 0.1E-3, 10.0))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (unitChan1, 0, 26))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (unitChan2, 0, 26))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViBooleanRange (triggerLevelUnit))
        return VI_ERROR_PARAMETER8;
    if (triggerLevelUnit == VI_TRUE)        
        if (rsupl_invalidViReal64Range (triggerLevel, -240.0, 0.0))
            return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 21))
        return VI_ERROR_INSTR_PARAMETER11;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, -1.0E3, 1.0E3))
            return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (notchGain, 0, 3))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (notchFreq, 0, 1))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViReal64Range (notchFreqValueHz, 10.0, 22.5E3))
        return VI_ERROR_INSTR_PARAMETER14;

    Fmt (buffer, "%s<:SENS:FUNC \'RMS\';:SENS:VOLT:APER:MODE %s", rmsMeasTimeArr[measTime]);
    bytesFmt = NumFmtdBytes ();
    if (measTime < 2) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:APER %fS", measTimeValues);
        bytesFmt += NumFmtdBytes ();
    }       
    if (measTime == 1) {
        Fmt (buffer, "%s[a]<;:TRIG:LEV %f%s", triggerLevel, triggerValUnitArr[triggerLevelUnit]);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s<:SENS:FUNC:SNS %s;:SENS:NOTC:STAT %s", stateArr[SNMeasure], rmsNotchGainArr[notchGain]);
        bytesFmt = NumFmtdBytes ();
        if (notchGain > 0) {
            Fmt (buffer, "%s[a]<;:SENS:NOTC:FREQ:MODE %s", rmsNotchFreqArr[notchFreq]);
            bytesFmt += NumFmtdBytes ();
            if (notchFreq == 0) {
                Fmt (buffer, "%s[a]<;FIX %fHZ", notchFreqValueHz);
                bytesFmt += NumFmtdBytes ();
            }
        }       
        if (SNMeasure == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SENS:VOLT:REF:MODE %s", rmsRefArr[reference]);
            bytesFmt += NumFmtdBytes ();
            if (reference == 0) {
                Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
                bytesFmt += NumFmtdBytes ();
            }
        }               
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    }           
    else {
        if (digMode == 0) {
            Fmt (buffer,"%s<:SENS:FUNC:SNS %s", stateArr[SNMeasure]);
            bytesFmt = NumFmtdBytes ();         
            if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
                return rsupl_status;        
        }           
        if (SNMeasure == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SENS:VOLT:REF:MODE %s", rmsRefArr[reference]);
            bytesFmt += NumFmtdBytes ();
            if (reference == 0) {
                Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
                bytesFmt += NumFmtdBytes ();
            }
        }               
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    }
    
    if (!(SNMeasure && (digMode == 0)))
        {
        if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
            return rsupl_status;
    
        switch (chanType) {
            case 1:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s", rmsUnitArr[unitChan1]);
                break;
            case 2:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT2 %s", rmsUnitArr[unitChan2]);
                break;
            default:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[unitChan1], rmsUnitArr[unitChan2]);
                break;
        }
        bytesFmt = NumFmtdBytes ();

        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;

        if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
            return rsupl_status;
        }
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: RMS Selective                                                   */
/* Purpose:  This function enables and configures measurement of AC voltages */
/*           of any desired waveform up to 30 V with narrow bandpass or      */
/*           bandstop. A superimposed DC is not measured.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncRMSSel (ViSession instrSession,
                    ViInt32 measTime, ViReal64 measTimeValues,
                    ViInt32 unitChan1, ViInt32 unitChan2, ViInt32 freqMode,
                    ViReal64 frequencyValueHz, ViReal64 genTracFactor, ViInt32 reference,
                    ViReal64 referenceValue, ViInt32 referenceValueUnit,
                    ViInt32 notchGain, ViInt32 bandwidth,
                    ViReal64 bandwidthValueHz)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 digMode = 0;
    ViInt32 chanType = 1;
    
    if (rsupl_invalidViInt32Range (measTime, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (measTimeValues, 0.1E-3, 10.0))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (unitChan1, 0, 25))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (unitChan2, 0, 25))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (freqMode, 0, 4))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViReal64Range (frequencyValueHz, 0.0, 110.0E3))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViReal64Range (genTracFactor, 1.0, 20.0))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 21))
        return VI_ERROR_INSTR_PARAMETER11;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, -1.0E3, 1.0E3))
            return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (notchGain, 0, 3))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (bandwidth, 0, 9))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViReal64Range (bandwidthValueHz, 0.0, 110.0E3))
        return VI_ERROR_INSTR_PARAMETER14;

    
    Fmt (buffer, "%s<:SENS:FUNC \'RMSS\';:SENS:VOLT:APER:MODE %s;"
                ":SENS:VOLT:REF:MODE %s;:SENS:BWID:MODE %s;:SENS:FREQ:MODE %s", 
        rmsMeasTimeArr[measTime], rmsRefArr[reference], 
        rmsBWidthModeArr[bandwidth], rmsFreqModeArr[freqMode]);
    bytesFmt = NumFmtdBytes ();
    if (measTime < 2) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:APER %f", measTimeValues);
        bytesFmt += NumFmtdBytes ();
    }       
    if (reference == 0) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if (freqMode == 0) {
        Fmt (buffer, "%s[a]<;:SENS:FREQ:FIX %fHZ", frequencyValueHz);
        bytesFmt += NumFmtdBytes ();
    } else if (freqMode == 1) {
        Fmt (buffer, "%s[a]<;:SENS:FREQ:FACT %f", genTracFactor);
        bytesFmt += NumFmtdBytes ();
    }
    
    if (bandwidth < 2) {
        Fmt (buffer, "%s[a]<;:SENS:BWID:RES %fHZ", bandwidthValueHz);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s<:SENS:NOTC:STAT %s", rmsNotchGainArr[notchGain]);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
    }           

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;
        
    switch (chanType) {
        case 1:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s", rmsUnitArr[unitChan1]);                
            break;
        case 2:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT2 %s", rmsUnitArr[unitChan2]);
            break;
        default:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[unitChan1], rmsUnitArr[unitChan2]);
            break;
    }
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: RMS Sweep                                                       */
/* Purpose:  This function enables and configures RMS Sweep measurement.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncRMSSwe (ViSession instrSession, ViInt32 spacing,
    ViReal64 startFreq, ViReal64 stopFreq, ViReal64 ptsStep, ViInt32 sweCtrl,
    ViInt32 stepUnit, ViString fileName)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (spacing, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (startFreq, 2.0, 120.0E3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (stopFreq, 2.0, 120.0E3))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (stepUnit, 0, 8))
        return VI_ERROR_PARAMETER7;
    switch (spacing) {
        case 2:
            if (stepUnit == 0)
                if (rsupl_invalidViReal64Range (ptsStep, 2.0, 110.0E3))
                    return VI_ERROR_PARAMETER5;
            break;
        case 3:
            if (rsupl_invalidViReal64Range (ptsStep, 1.0, 200.0))
                return VI_ERROR_PARAMETER5;
            break;
        default:
            if (rsupl_invalidViReal64Range (ptsStep, 2.0, 1024.0))
                return VI_ERROR_PARAMETER5;
            break;
    }
    if (rsupl_invalidViInt32Range (sweCtrl, 0, 5))
        return VI_ERROR_PARAMETER6;
    
    if ((sweCtrl == 0) || (sweCtrl == 5)) {
        Fmt (buffer, "%s<:SENS:FREQ:MODE %s", rmsSweCtrlArr[sweCtrl]);
        bytesFmt = NumFmtdBytes ();
    }
    else {  
        Fmt (buffer, "%s<:SENS:FREQ:MODE %s", rmsSweCtrlArr[sweCtrl]);
        bytesFmt = NumFmtdBytes ();
        if ((sweCtrl == 2) || (sweCtrl == 4)) {
            Fmt (buffer, "%s[a]<;:MMEM:LOAD:LIST FREQ, \"%s\"", fileName);
            bytesFmt += NumFmtdBytes ();
        }
        else {
            Fmt (buffer, "%s[a]<;:SENS:FREQ:STAR %fHZ;STOP %fHZ;:SENS:SWE:SPAC %s", 
                startFreq, stopFreq, sweSpacingArr[spacing]);
            bytesFmt += NumFmtdBytes ();
            if (spacing < 2) {
                Fmt (buffer, "%s[a]< %d[b4]", (int)ptsStep);
                bytesFmt += NumFmtdBytes ();
            }       
            else {
                Fmt (buffer, "%s[a]< %f", ptsStep);
                bytesFmt += NumFmtdBytes ();
                if (spacing == 2) {
                    Fmt (buffer, "%s[a]<%s", unitFreqArr[stepUnit]);
                    bytesFmt += NumFmtdBytes ();
                }
            }           
        }
    }   
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Peak & S/N                                                      */
/* Purpose:  This function enables and configures Peak & S/N measurement. In */
/*           this measurement the peak value detector follows the waveform   */
/*           without delay.  The signal-to-noise measurement is also         */
/*           available.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncPeak (ViSession instrSession,
                    ViBoolean SNMeasure, ViInt32 measMode, ViInt32 unitChan1,
                    ViInt32 unitChan2, ViInt32 intervalTime,
                    ViReal64 intervalTimeValues, ViInt32 reference,
                    ViReal64 referenceValue, ViInt32 referenceValueUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType = 1;
    ViInt32 digMode = 0;
    
    if (rsupl_invalidViBooleanRange (SNMeasure))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (measMode, 0, 3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (unitChan1, 0, 26))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (unitChan2, 0, 26))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (intervalTime, 0, 3))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViReal64Range (intervalTimeValues, 0.02, 100.0))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 21))
        return VI_ERROR_INSTR_PARAMETER10;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, -1.0E3, 1.0E3))
            return VI_ERROR_INSTR_PARAMETER9;
    
    Fmt (buffer, "%s<:SENS:FUNC \'PEAK\';:SENS:FUNC:MMOD %s;"
                ":SENS:VOLT:INTV:MODE %s", 
        peakMeasModeArr[measMode], intervalTimeArr[intervalTime]);
    bytesFmt = NumFmtdBytes ();
    if (intervalTime == 0) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:INTV %fS", intervalTimeValues);
        bytesFmt += NumFmtdBytes ();
    }       

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
        
    if (((instrType == VI_TRUE) && (digMode == 0)) || (instrType == VI_FALSE)) {
        Fmt (buffer,"%s<:SENS:FUNC:SNS %s", stateArr[SNMeasure]);
        bytesFmt = NumFmtdBytes ();         
        if (SNMeasure == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SENS:VOLT:REF:MODE %s", rmsRefArr[reference]);
            bytesFmt += NumFmtdBytes ();
            if (reference == 0) {
                Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
                bytesFmt += NumFmtdBytes ();
            }
        }               
    }
    else {
        Fmt (buffer, "%s<:SENS:VOLT:REF:MODE %s", rmsRefArr[reference]);
        bytesFmt = NumFmtdBytes ();
        if (reference == 0) {
            Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
            bytesFmt += NumFmtdBytes ();
        }
    }
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
    
    if (!(SNMeasure && (digMode == 0)))
        {
        if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
            return rsupl_status;
        
        switch (chanType) {
            case 1:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s", rmsUnitArr[unitChan1]);                
                break;
            case 2:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT2 %s", rmsUnitArr[unitChan2]);
                break;
            default:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[unitChan1], rmsUnitArr[unitChan2]);
                break;
        }
        if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
            return rsupl_status;
        }
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Quasi-Peak & S/N                                                */
/* Purpose:  This function enables and configures Quasi-Peak & S/N           */
/*           measurement. The peak value is detected with subsequent and     */
/*           defined rising and falling times. The signal-to-noise           */
/*           measurement is also available.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncQPeak (ViSession instrSession,
                    ViBoolean SNMeasure, ViInt32 unitChan1, ViInt32 unitChan2,
                    ViInt32 intervalTime, ViReal64 intervalTimeValues,
                    ViInt32 reference, ViReal64 referenceValue,
                    ViInt32 referenceValueUnit, ViInt32 notchGain,
                    ViInt32 notchFreq, ViReal64 notchFreqValueHz)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType = 1;
    ViInt32 digMode = 0;
    
    if (rsupl_invalidViBooleanRange (SNMeasure))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (unitChan1, 0, 19))
        if (rsupl_invalidViInt32Range (unitChan1, 26, 26))
            return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (unitChan2, 0, 19))
        if (rsupl_invalidViInt32Range (unitChan1, 26, 26))
            return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (intervalTime, 0, 4))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViReal64Range (intervalTimeValues, 0.02, 100.0))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 15))
        return VI_ERROR_INSTR_PARAMETER9;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, -1.0E3, 1.0E3))
            return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (notchGain, 0, 3))
        return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (notchFreq, 0, 1))
        return VI_ERROR_INSTR_PARAMETER11;
    if (rsupl_invalidViReal64Range (notchFreqValueHz, 10.0, 22.5E3))
        return VI_ERROR_INSTR_PARAMETER12;

    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
    
    if (digMode > 0) {
        rsupl_status = VI_RSUPL_ERROR_ANALYZER_FUNCTION;
        return rsupl_status;
    }
    
    Fmt (buffer, "%s<:SENS:FUNC \'QPE\';:SENS:VOLT:INTV:MODE %s", intervalTimeArr[intervalTime]);
    bytesFmt = NumFmtdBytes ();
    if (intervalTime == 0) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:INTV %fS", intervalTimeValues);
        bytesFmt += NumFmtdBytes ();
    }       

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
    
    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s<:SENS:FUNC:SNS %s;:SENS:NOTC:STAT %s", stateArr[SNMeasure], rmsNotchGainArr[notchGain]);
        bytesFmt = NumFmtdBytes ();
        if (notchGain > 0) {
            Fmt (buffer, "%s[a]<;:SENS:NOTC:FREQ:MODE %s", rmsNotchFreqArr[notchFreq]);
            bytesFmt += NumFmtdBytes ();
            if (notchFreq == 0) {
                Fmt (buffer, "%s[a]<;FIX %fHZ", notchFreqValueHz);
                bytesFmt += NumFmtdBytes ();
            }
        }       
        if (SNMeasure == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SENS:VOLT:REF:MODE %s", rmsRefArr[reference]);
            bytesFmt += NumFmtdBytes ();
            if (reference == 0) {
                Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
                bytesFmt += NumFmtdBytes ();
            }
        }               
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    }   
    
    else {
        Fmt (buffer,"%s<:SENS:FUNC:SNS %s", stateArr[SNMeasure]);
        bytesFmt = NumFmtdBytes ();         
        if (SNMeasure == VI_FALSE) {
            Fmt (buffer, "%s[a]<;:SENS:VOLT:REF:MODE %s", rmsRefArr[reference]);
            bytesFmt += NumFmtdBytes ();
            if (reference == 0) {
                Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
                bytesFmt += NumFmtdBytes ();
            }
        }                   
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    }
    
    if (!SNMeasure)
        {
        if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
            return rsupl_status;
        
        switch (chanType) {
            case 1:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s", rmsUnitArr[unitChan1]);                
                break;
            case 2:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT2 %s", rmsUnitArr[unitChan2]);
                break;
            default:
                Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[unitChan1], rmsUnitArr[unitChan2]);
                break;
        }
        if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
            return rsupl_status;
        }
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: DC                                                              */
/* Purpose:  This function enables and configures the DC voltage measurement.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncDC (ViSession instrSession,
                    ViInt32 measTime, ViInt32 unitChan1, ViInt32 unitChan2,
                    ViInt32 reference, ViReal64 referenceValue,
                    ViInt32 referenceValueUnit, ViReal64 measTimeValues)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType = 1;
    ViInt32 digMode = 0;
    
    if (rsupl_invalidViInt32Range (unitChan1, 0, 19))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (unitChan2, 0, 19))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (measTime, 0, 1))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViReal64Range (measTimeValues, 0.0, 1.5))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 15))
        return VI_ERROR_PARAMETER8;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, -1.0E3, 1.0E3))
            return VI_ERROR_PARAMETER7;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
    
    if (digMode > 0) {
        rsupl_status = VI_RSUPL_ERROR_ANALYZER_FUNCTION;
        return rsupl_status;
    }
    
    Fmt (buffer, "%s<:SENS:FUNC \'DC\';:SENS:VOLT:REF:MODE %s;"
                ":SENS:VOLT:APER:MODE %s", 
        rmsRefArr[reference], dcMeasTimeArr[measTime]);
    bytesFmt = NumFmtdBytes ();
    if (measTime == 0) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:APER %fS", measTimeValues);
        bytesFmt += NumFmtdBytes ();
    }       
    if (reference == 0) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;
        
    switch (chanType) {
        case 1:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s", rmsUnitArr[unitChan1]);                
            break;
        case 2:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT2 %s", rmsUnitArr[unitChan2]);
            break;
        default:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[unitChan1], rmsUnitArr[unitChan2]);
            break;
    }
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Total Harmonic Distortion                                       */
/* Purpose:  This function enables and configures the Total Harmonics        */
/*           Distortion measurement.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncTHD (ViSession instrSession,
                    ViBoolean fundamentalFreq,
                    ViReal64 fundamentalFreqValueHz, ViInt32 chanUnit,
                    ViBoolean dynamicMode, ViInt32 measMode,
                    ViReal64 referenceValue, ViInt32 referenceValueUnit,
                    ViInt32 harmonics)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViByte buffer[BUFFER_SIZE];
    ViInt32 chanType, digMode = 0;
    
    if (rsupl_invalidViBooleanRange (fundamentalFreq))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (fundamentalFreqValueHz, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (chanUnit, 0, 1))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViBooleanRange (dynamicMode))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (measMode, 0, 7))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (harmonics, 0, 255))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 15))
        return VI_ERROR_INSTR_PARAMETER9;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, 0.0, 100.0))
            return VI_ERROR_PARAMETER8;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
    
    if (digMode > 0)
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;

    Fmt (buffer, "%s<:SENS:FUNC \'THD\';:SENS:FUNC:MMOD %s;"
                ":SENS:VOLT:FUND:MODE %s", 
        thdMeasModeArr[measMode], thdFundModeArr[fundamentalFreq]);
    bytesFmt = NumFmtdBytes ();
    
    if (measMode > 3) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
        bytesFmt += NumFmtdBytes ();
    }       
    if ((measMode == 0) || (measMode == 4)) {
        Fmt (buffer, "%s[a]<;:SENS:FUNC:DIST %d[b4]", harmonics);
        bytesFmt += NumFmtdBytes ();
    }       
    if (fundamentalFreq == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:FUND %fHZ", fundamentalFreqValueHz);
        bytesFmt += NumFmtdBytes ();
    }
    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s[a]<;:SENS:FUNC:DMOD %s;", thdDynModeArr[dynamicMode]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;

    Fmt (buffer, "%s[a]<:SENS:UNIT%s %s;", ((chanType == 2) ? "2": ""), thdUnitNewArr[chanUnit]);
    bytesFmt += NumFmtdBytes ();

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Total Harmonic Distortion & Noise / SINAD                       */
/* Purpose:  This function enables and configures the Total Harmonics        */
/*           Distortion and Noise measurement.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncTHDN (ViSession instrSession,
                    ViBoolean fundamentalFreq,
                    ViReal64 fundamentalFreqValueHz, ViInt32 chanUnit,
                    ViBoolean dynamicMode, ViInt32 measMode,
                    ViBoolean rejection, ViReal64 freqLimitLowHz,
                    ViReal64 freqLimitHighHz, ViReal64 referenceValue,
                    ViInt32 referenceValueUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType,digMode = 0;
    
    if (rsupl_invalidViBooleanRange (fundamentalFreq))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (fundamentalFreqValueHz, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (chanUnit, 0, 21))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViBooleanRange (dynamicMode))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (measMode, 0, 4))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViBooleanRange (rejection))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViReal64Range (freqLimitLowHz, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViReal64Range (freqLimitHighHz, 0.0, 125.0E3))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 15))
        return VI_ERROR_INSTR_PARAMETER11;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, 0.0, 100.0))
            return VI_ERROR_INSTR_PARAMETER10;

    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
    
    if (digMode > 0)
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;

    Fmt (buffer, "%s<:SENS:FUNC \'THDN\';:SENS:FUNC:MMOD %s;:SENS:UNIT%s %s;"
                ":SENS:VOLT:FUND:MODE %s;:SENS:FREQ:LIM:UPP %fHZ;LOW %fHZ", 
        thdnMeasModeArr[measMode], ((chanType == 2)? "2":""), thdUnitArr[chanUnit], thdFundModeArr[fundamentalFreq], 
        freqLimitHighHz, freqLimitLowHz);
    bytesFmt = NumFmtdBytes ();
    
    if (measMode > 2) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
        bytesFmt += NumFmtdBytes ();
    }       
    if (fundamentalFreq == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:FUND %fHZ", fundamentalFreqValueHz);
        bytesFmt += NumFmtdBytes ();
    }
    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s[a]<;:SENS:FUNC:DMOD %s;", thdDynModeArr[dynamicMode]);
        bytesFmt += NumFmtdBytes ();
        if (dynamicMode == VI_TRUE) {
            Fmt (buffer, "%s[a]<;:SENS:THDN:REJ %s;", thdnRejectionArr[rejection]);
            bytesFmt += NumFmtdBytes ();
        }
    }
    if (instrType == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SENS:THDN:REJ %s;", thdnRejectionArr[rejection]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Modulation Distortion                                           */
/* Purpose:  This function enables and configures the Modulation Distortion  */
/*           measurement.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncModDist (ViSession instrSession,
                    ViInt16 chanUnit, ViBoolean dynamicMode)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType, digMode = 0;
    
    if (rsupl_invalidViInt32Range (chanUnit, 20, 21))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (dynamicMode))
        return VI_ERROR_PARAMETER3;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
    
    if (digMode > 0)
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;

    Fmt (buffer, "%s<:SENS:FUNC \'MDIS\';:SENS:UNIT%s %s", ((chanType == 2)? "2":""), thdUnitArr[chanUnit]);
    bytesFmt = NumFmtdBytes ();
    
    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s[a]<;:SENS:FUNC:DMOD %s;", thdDynModeArr[dynamicMode]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Difference Frequency Distortion                                 */
/* Purpose:  This function enables and configures the Difference Frequency   */
/*           Distortion measurement.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncDFD (ViSession instrSession,
                    ViInt32 chanUnit, ViBoolean dynamicMode, ViInt32 measMode)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType, digMode = 0;
    
    if (rsupl_invalidViInt32Range (chanUnit, 20, 21))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (dynamicMode))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (measMode, 0, 3))
        return VI_ERROR_PARAMETER4;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if (digMode > 0)
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;

    Fmt (buffer, "%s<:SENS:FUNC \'DFD\';:SENS:UNIT%s %s;FUNC:MMOD %s", 
        ((chanType == 2) ? "2" : ""), thdUnitArr[chanUnit], dfdMeasModeArr[measMode]);
    bytesFmt = NumFmtdBytes ();
    if (instrType == VI_FALSE) {
        if (measMode < 2) {
            Fmt (buffer, "%s[a]<;:SENS:FUNC:DMOD %s;", thdDynModeArr[dynamicMode]);
            bytesFmt += NumFmtdBytes ();
        }               
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Wow & Flutter                                                   */
/* Purpose:  This function enables and configures the Wow and Flutter        */
/*           measurement. The frequency deviation of a recording device is   */
/*           measured.                                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncWF (ViSession instrSession,
                    ViBoolean weighting, ViInt32 standard)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 digMode = 0;
    
    if (rsupl_invalidViBooleanRange (weighting))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (standard, 0, 4))
        return VI_ERROR_PARAMETER3;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if (digMode > 0 && instrType)
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;

    Fmt (buffer, "%s<:SENS:FUNC \'WAF\';FUNC:WEIG %s;STAN %s;:SENS:UNIT PCT", 
        stateArr[weighting], wfStandardArr[standard]);
        
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Polarity                                                        */
/* Purpose:  This function enables and configures the Polarity measurement.  */
/*           The analyzer weights and displays the polarity of the DUT output*/
/*           signal.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncPol (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 digMode = 0;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if (digMode > 0) {
        rsupl_status = VI_RSUPL_ERROR_ANALYZER_FUNCTION;
        return rsupl_status;
    }
    
    Fmt (buffer, "%s<:SENS:FUNC \'POL\'");
        
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: FFT                                                             */
/* Purpose:  This function configures the FFT function of the analyzer. It   */
/*           transforms the unfiltered input signal into a frequency domain  */
/*           using fast fourier transform algorithm.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncFFT (ViSession instrSession,
                    ViBoolean averagingMode, ViInt32 zoom,
                    ViReal64 centerFreqHz, ViInt32 unitChan1,
                    ViInt32 unitChan2, ViInt32 FFTSize,
                    ViInt32 averagingCount, ViInt32 reference,
                    ViReal64 referenceValue, ViInt32 referenceValueUnit,
                    ViInt32 window, ViInt32 notchGain, ViInt32 notchFreq,
                    ViReal64 notchFreqValueHz, ViReal64 betaFactor)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType = 1;
    ViInt32 digMode = 0;
    
    if (rsupl_invalidViBooleanRange (averagingMode))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (zoom, 0, 7))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViReal64Range (centerFreqHz, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (unitChan1, 0, 25))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViInt32Range (unitChan2, 0, 25))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (FFTSize, 0, 6))
        return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViInt32Range (averagingCount, 1, 256))
        return VI_ERROR_PARAMETER8;
    if (rsupl_invalidViInt32Range (reference, 0, 6))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (referenceValueUnit, 0, 21))
        return VI_ERROR_INSTR_PARAMETER11;
    if (referenceValueUnit == 0)
        if (rsupl_invalidViReal64Range (referenceValue, -1.0E3, 1.0E3))
            return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (window, 0, 8))
        return VI_ERROR_INSTR_PARAMETER12;
    if (rsupl_invalidViInt32Range (notchGain, 0, 3))
        return VI_ERROR_INSTR_PARAMETER13;
    if (rsupl_invalidViInt32Range (notchFreq, 0, 1))
        return VI_ERROR_INSTR_PARAMETER14;
    if (rsupl_invalidViReal64Range (notchFreqValueHz, 10.0, 22.5E3))
        return VI_ERROR_INSTR_PARAMETER15;
    if (rsupl_invalidViReal64Range (betaFactor, 1.0, 20.0))
        return VI_ERROR_INSTR_PARAMETER16;

    Fmt (buffer, "%s<:SENS:FUNC \'FFT\';:SENS:VOLT:REF:MODE %s;"
                ":CALC:TRAN:FREQ:WIND %s;ZOOM %s;FFT %s;AVER %d[b4];AVER:TCON %s", 
        rmsRefArr[reference], funcFFTWindowArr[window], FFTZoomArr[zoom], FFTSizeArr[FFTSize], 
        averagingCount, FFTAverModeArr[averagingMode]);
    bytesFmt = NumFmtdBytes ();
    if (reference == 0) {
        Fmt (buffer, "%s[a]<;:SENS:VOLT:REF %f%s", referenceValue, referenceValUnitArr[referenceValueUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    if (zoom > 0) {
        Fmt (buffer, "%s[a]<;:CALC:TRAN:FREQ:CENT %fHZ", centerFreqHz);
        bytesFmt += NumFmtdBytes ();
    }
    if (window == 8) {
        Fmt (buffer, "%s[a]<;WIND:BETA %f", betaFactor);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
        
    if (instrType == VI_FALSE) {
        Fmt (buffer, "%s<:SENS:NOTC:STAT %s", rmsNotchGainArr[notchGain]);
        bytesFmt = NumFmtdBytes ();
        if (notchGain > 0) {
            Fmt (buffer, "%s[a]<;:SENS:NOTC:FREQ:MODE %s", rmsNotchFreqArr[notchFreq]);
            bytesFmt += NumFmtdBytes ();
            if (notchFreq == 0) {
                Fmt (buffer, "%s[a]<;FIX %fHZ", notchFreqValueHz);
                bytesFmt += NumFmtdBytes ();
            }
        }       
        if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
            return rsupl_status;        
    }           

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;
        
    switch (chanType) {
        case 1:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s", rmsUnitArr[unitChan1]);                
            break;
        case 2:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT2 %s", rmsUnitArr[unitChan2]);
            break;
        default:
            Fmt (buffer, "%s<:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[unitChan1], rmsUnitArr[unitChan2]);
            break;
    }
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Waveform                                                        */
/* Purpose:  This function configures the time-domain acquisition and display*/
/*           of the input signal.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncWfm (ViSession instrSession,
                    ViBoolean measMode, ViInt32 compressionFactor,
                    ViInt32 interpolation, ViInt32 chanUnit,
                    ViReal64 traceLengths, ViReal64 referenceLevel,
                    ViInt32 referenceLevelUnit, ViBoolean triggerSlope,
                    ViReal64 triggerLevel, ViInt32 triggerLevelUnit,
                    ViInt32 triggerSource)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType;
    ViInt32 digMode = 0;
    
    if (rsupl_invalidViBooleanRange (measMode))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (compressionFactor, 2, 1024))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (interpolation, 0, 5))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViInt32Range (chanUnit, 0, 25))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViReal64Range (traceLengths, 0.0, 100.0E5))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViInt32Range (referenceLevelUnit, 0, 21))
        return VI_ERROR_PARAMETER8;
    if (referenceLevelUnit == 0)
        if (rsupl_invalidViReal64Range (referenceLevel, -1.0E3, 1.0E3))
            return VI_ERROR_PARAMETER7;
    if (rsupl_invalidViBooleanRange (triggerSlope))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsupl_invalidViInt32Range (triggerLevelUnit, 0, 21))
        return VI_ERROR_INSTR_PARAMETER11;
    if (referenceLevelUnit == 0)
        if (rsupl_invalidViReal64Range (triggerLevel, -50.0, 50.0))
            return VI_ERROR_INSTR_PARAMETER10;
    if (rsupl_invalidViInt32Range (triggerSource, 0, 2))
        return VI_ERROR_INSTR_PARAMETER12;

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;
    Fmt (buffer, "%s<:SENS:FUNC \'WAV\';:SENS:VOLT:REF %f%s;"
                ":SENS:FUNC:MMOD %s;:TRIG:LEV %f%s;SLOP %s;:SENS:WAV:DUR %fS",  
        referenceLevel, referenceValUnitArr[referenceLevelUnit], 
        wfmMeasModeArr[measMode], triggerLevel, referenceValUnitArr[triggerLevelUnit], 
        trigSlopeArr[triggerSlope], traceLengths);
    bytesFmt = NumFmtdBytes ();
    if (chanType > 0) {
        if (chanType > 2)
            Fmt (buffer, "%s[a]<;:SENS:VOLT:UNIT1 %s;UNIT2 %s", rmsUnitArr[chanUnit], rmsUnitArr[chanUnit]);
        else
            Fmt (buffer, "%s[a]<;:SENS:VOLT:UNIT%d[b4] %s", chanType, rmsUnitArr[chanUnit]);
        
        bytesFmt += NumFmtdBytes ();
    }
    
    if (measMode == VI_TRUE) {
        Fmt (buffer, "%s[a]<;:SENS:WAV:COMP %d[b4]", compressionFactor);
        bytesFmt += NumFmtdBytes ();
    }
    if (measMode == VI_FALSE) {
        Fmt (buffer, "%s[a]<;:SENS:SMO:APER %s", smoothingArr[interpolation]);
        bytesFmt += NumFmtdBytes ();
    }
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
        
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        
        
    if (((instrType == VI_TRUE) && (digMode == 0)) || (instrType == VI_FALSE)) {
        Fmt (buffer, "%s<:TRIG:CHAN %s", trigSourceArr[triggerSource]);
        if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
            return rsupl_status;        
    }
  
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Filter Simulation                                               */
/* Purpose:  This function configures the filter simulation function.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncFiltSim (ViSession instrSession,
                    ViInt32 unit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 chanType, digMode = 0;

    if (rsupl_invalidViInt32Range (unit, 20, 21))
        return VI_ERROR_PARAMETER2;

    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if ((digMode > 0) && (instrType == VI_TRUE))
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;

    if ((rsupl_status = rsupl_getAnalyzerActiveChannels (instrSession, &chanType)) < 0)
        return rsupl_status;

    Fmt (buffer, "%s<:SENS:FUNC \'FILT\';:SENS:VOLT:UNIT%s %s",
        ((chanType == 2) ? "2" : ""), thdUnitArr[unit]);

    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
  
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Protocol                                                        */
/* Purpose:  This function configures the analysis and display of AES        */
/*           protocol data as well as measurement of the input peak and the  */
/*           sample frequency.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confAnalyzerFuncProt (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViBoolean instrType = VI_FALSE;
    ViInt32 digMode = 0;
    
    if ((rsupl_status = rsupl_getAnalyzerMode (instrSession, &digMode, &instrType)) < 0)
        return rsupl_status;        

    if ((digMode > 0) || (instrType == VI_FALSE)) {
        return VI_RSUPL_ERROR_ANALYZER_FUNCTION;
    }
    
    Fmt (buffer, "%s<:SENS:FUNC \'PROT\'");
        
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Store/Load Instrument Setup                                     */
/* Purpose:  This function saves or recalles instrument's setup.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confSaveRecallSetup (ViSession instrSession, 
                    ViBoolean stpMode, ViString fileName, ViBoolean fileAttr,
                    ViBoolean stpType)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViBooleanRange (stpMode))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (fileAttr))
        return VI_ERROR_PARAMETER4;
    if (rsupl_invalidViBooleanRange (stpType))
        return VI_ERROR_PARAMETER5;

    Fmt (buffer, "%s<:MMEM:%s:STAT %s,\"%s\"",stpModeArr[stpMode], stpTypeArr[stpType], fileName);
    bytesFmt = NumFmtdBytes ();
    
    if (stpMode == VI_FALSE) {
        Fmt (buffer, "%s[a]<;:MMEM:STOR:STAT:RONL %s", stateArr[fileAttr]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
                
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Instrument Version                                              */
/* Purpose:  This function returns the instrument version numbers.           */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrVer (ViSession instrSession, ViChar _VI_FAR swVer[],
                    ViChar _VI_FAR stpVer[], ViChar _VI_FAR anaBoardVer[], 
                    ViChar _VI_FAR cpuVer[], ViChar _VI_FAR digBoardVer[])
{
    ViStatus rsupl_status = VI_SUCCESS;
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:SOFT:VERS? SOFT", swVer)) < 0)
        return rsupl_status;    
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:SOFT:VERS? SETU", stpVer)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:AHAR:VERS? ABO", anaBoardVer)) < 0)
        return rsupl_status;    

    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:DHAR:VERS? CPU", cpuVer)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:DHAR:VERS? DBO", digBoardVer)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Instrument Options                                              */
/* Purpose:  This function returns the list of installed instrument options. */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrOpt (ViSession instrSession, ViChar _VI_FAR b4[], 
                    ViChar _VI_FAR b2[], ViChar _VI_FAR b21[], ViChar _VI_FAR b1[], 
                    ViChar _VI_FAR b5[], ViChar _VI_FAR b22[], ViChar _VI_FAR b10[])
{
    ViStatus rsupl_status = VI_SUCCESS;
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? LDG", b1)) < 0)
        return rsupl_status;    
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? REMO", b4)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? DAUD", b2)) < 0)
        return rsupl_status;    

    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? SPE", b5)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? DAPR", b21)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? DAJ", b22)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":SYST:OPT:VERS? SQCO", b10)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Instrument Trigger Config                                       */
/* Purpose:  This function configures the instrument's trigger.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrTrig (ViSession instrSession, ViBoolean trigMode)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViBooleanRange (trigMode))
        return VI_ERROR_PARAMETER2;

    Fmt (buffer, "%s<:INIT:CONT %s;*WAI", stateArr[trigMode]);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
                
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Audio Monitor                                                   */
/* Purpose:  This function sets Audio Monitor Mode. Only available with      */
/*           option B5 installed.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrAudioMonitor (ViSession instrSession,
                                               ViBoolean speakerState,
                                               ViInt32 speakerSource,
                                               ViBoolean phonesMode,
                                               ViReal64 speakerGain,
                                               ViReal64 speakerVolume)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rsupl_invalidViBooleanRange (speakerState))
        return VI_ERROR_PARAMETER2;
    
    if (speakerState) {
        if (rsupl_invalidViInt32Range (speakerSource, 0, 9))
            return VI_ERROR_PARAMETER3;
        if (rsupl_invalidViBooleanRange (phonesMode))
            return VI_ERROR_PARAMETER4;
        if (rsupl_invalidViReal64Range (speakerGain, -120.0, 120.0))
            return VI_ERROR_PARAMETER5;
        if (rsupl_invalidViReal64Range (speakerVolume, 0.0, 100.0))
            return VI_ERROR_PARAMETER6;
        Fmt (buffer, "%s<\n:SYST:SPE:SOUR %s", speakerSourceArr[speakerSource]);
        bytesFmt += NumFmtdBytes ();
        if (speakerSource > 3 && speakerSource < 7) {
            Fmt (buffer, "%s[a]<;:SYST:SPE:GAIN %f", speakerGain);
            bytesFmt += NumFmtdBytes ();
        }
        Fmt (buffer, "%s[a]<;:SYST:SPE:VOL %f", speakerVolume);
        bytesFmt += NumFmtdBytes ();
        Fmt (buffer, "%s[a]<;:SYST:PHON %s", (phonesMode)?"PERM":"SPKC");
        bytesFmt += NumFmtdBytes ();
        Fmt (buffer, "%s[a]<;:SYST:SPE:STAT ON\n");
        bytesFmt += NumFmtdBytes ();
    } else {
        Fmt (buffer, "%s<\n:SYST:SPE:STAT OFF\n");
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
            
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  This function triggers the instrument to perform a              */
/*           self-calibration of selected components.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrCalibr (ViSession instrSession, ViInt32 calibGen,
                    ViInt32 calibAnl, ViInt32 calibDig)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViInt32Range (calibGen, 0, 1))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (calibAnl, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViInt32Range (calibDig, 0, 1))
        return VI_ERROR_PARAMETER4;

    Fmt (buffer, "%s<:CAL:LDG:AUTO %s;:CAL:ZERO:AUTO %s;:CAL:JITT:AUTO %s",
        calibArr[calibGen], calibArr[calibAnl], calibArr[calibDig]);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
                
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Display                                                         */
/* Purpose:  Turns on or off the instrument's numeric display, turns off     */
/*           cursors in the instrument's graph display. Useful to increase   */
/*           measurement speed.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrDisplay (ViSession instrSession,
                                          ViBoolean display, ViBoolean cursors)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rsupl_invalidViBooleanRange (display))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (cursors))
        return VI_ERROR_PARAMETER3;

    Fmt (buffer, "%s<:DISP:ANN %s", stateArr[display]);
    bytesFmt += NumFmtdBytes ();

    if (!cursors) {
        Fmt (buffer, "%s[a]<;:DISP:TRAC:CURS1:MODE OFF;:DISP:TRAC:CURS2:MODE OFF");
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;        
                
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Sense                                                           */
/* Purpose:  This VI is useful to switch off sense systems not needed,       */
/*           in order to increase measurement speed. Use the appropriate     */
/*           Config Analyzer VI to reselect a sense mode.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrSense (ViSession instrSession,
                                        ViBoolean sense1, ViBoolean sense2,
                                        ViBoolean sense3)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rsupl_invalidViBooleanRange (sense1))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViBooleanRange (sense2))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (sense3))
        return VI_ERROR_PARAMETER4;
    
    if (!sense1) {
        Fmt (buffer, "%s<\n:SENS1:FUNC 'OFF'");
    } else {
        Fmt (buffer, "%s<\n");
    }
    bytesFmt += NumFmtdBytes ();
    
    if (!sense2) {
        Fmt (buffer, "%s[a]<;:SENS2:FUNC 'OFF'");
        bytesFmt += NumFmtdBytes ();
    }
    
    if (!sense3) {
        Fmt (buffer, "%s[a]<;:SENS3:FUNC 'OFF'");
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
            
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;

    return rsupl_status;
}
     
/*===========================================================================*/
/* Function: Store Data Config                                               */
/* Purpose:  This function configures the data storing capabality of the     */
/*           instrument.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confStoreData (ViSession instrSession, ViInt32 dataType,
                    ViReal64 normFreq, ViString fileName, 
                    ViBoolean dataFormat, ViBoolean levSource, ViBoolean invert)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 bytesFmt = 0;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViInt32Range (dataType, 0, 9))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViReal64Range (normFreq, 0.0, 125.0E3))
        return VI_ERROR_PARAMETER3;
    if (rsupl_invalidViBooleanRange (dataFormat))
        return VI_ERROR_PARAMETER5;
    if (rsupl_invalidViBooleanRange (levSource))
        return VI_ERROR_PARAMETER6;
    if (rsupl_invalidViBooleanRange (invert))
        return VI_ERROR_PARAMETER7;
    
    Fmt (buffer, "%s<:MMEM:STOR:%s, \"%s\";FORM %s", storeParamArr[dataType],
        fileName, dataFormatArr[dataFormat]);
    bytesFmt = NumFmtdBytes ();
    if (dataType == 9) {
        Fmt (buffer, "%s[a]<;:CALC:EQU:FEED %s;NORM %fHZ;INV %s", dataLevSourceArr[levSource], 
            normFreq, stateArr[invert]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}


/*===========================================================================*/
/* Function: Screen To File Config                                           */
/* Purpose:  This function configures the instrument's front panel screen    */
/*           saving.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_confInstrScreenToFile (ViSession instrSession,
    ViBoolean fileType, ViString fileName)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if (rsupl_invalidViBooleanRange (fileType))
        return VI_ERROR_PARAMETER2;

    Fmt (buffer, "%s<:HCOPY:DEST %s, \"%s\"", hcopyDestArr[fileType], fileName);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
                
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Delete File                                                     */
/* Purpose:  This function deletes a file on the instrument's hard or floppy */
/*           drive.                                                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_dataStoreDel (ViSession instrSession, ViString fileName)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    Fmt (buffer, "%s<:MMEM:DEL \"%s\"", fileName);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Copy File                                                       */
/* Purpose:  This function copies one file on the instrument's hard or floppy*/
/*           drive to another.                                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_dataStoreCopy (ViSession instrSession, ViString sourceFileName,
    ViString targetFileName)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    Fmt (buffer, "%s<:MMEM:COPY \"%s\",\"%s\"", sourceFileName, targetFileName);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Select Working Directory                                        */
/* Purpose:  This function selects the working directory.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_dataStoreDir (ViSession instrSession, ViString path)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    Fmt (buffer, "%s<:MMEM:CDIR \"%s\"", path);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
   
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Read Block Data                                                 */
/* Purpose:  This function reads a block of data from the instrument and     */
/*           stores it into an array.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_dataReadBlock (ViSession instrSession, ViInt32 variable,
    ViReal64 blockData[], ViPInt32 numbOfVals)
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 length = 0;
    ViInt32 lengthOfLength = 0;

    if (rsupl_invalidViInt32Range (variable, 0, 19))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<:%s", blockReadPointsVarArr[variable]);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%d[b4]", &length) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if (length < 1)
        {
        *numbOfVals = 0;
        return VI_RSUPL_ERROR_BLOCK_DATA;
        }
    
    Fmt (buffer, "%s<:%s", blockReadVarArr[variable]);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = viRead (instrSession, buffer, 2, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>#%d[b4]", &lengthOfLength) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if ((rsupl_status = viRead (instrSession, buffer, lengthOfLength, &retCnt)) < 0)
        return rsupl_status;
        
    if (Scan (buffer, "%s>%d[b4]", &length) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if ((rsupl_status = viRead (instrSession, longBuffer, length+1, &retCnt)) < 0)
        return rsupl_status;
        
    *numbOfVals = length/4;
                        
    if (Scan (longBuffer, "%*f[zb4]>%*f[b8]", *numbOfVals, *numbOfVals, blockData) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Write Block Data                                                */
/* Purpose:  This function writes a block of data into the instrument.       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_dataWriteBlock (ViSession instrSession, ViInt32 variable, 
    ViReal64 blockData[], ViInt32 numbOfVals)
{
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 i = 0;

    if (rsupl_invalidViInt32Range (variable, 0, 18))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (numbOfVals, 0, 1023))
        return VI_ERROR_PARAMETER3;
        
    Fmt (longBuffer, "%s<:%s ", blockWriteVarArr[variable]);
    bytesFmt = NumFmtdBytes ();

    for (i = 0; i < numbOfVals; i++)
        {
        Fmt (longBuffer, "%s[a]<%f[b4],", ((ViReal32)blockData[i]));
        bytesFmt += NumFmtdBytes ();
        }
        
    if ((rsupl_status = viWrite (instrSession, longBuffer, bytesFmt-1, &retCnt)) < 0)
        return rsupl_status;        
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Read Single Channel                                             */
/* Purpose:  This function reads a measurement from the instrument.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_measSingleCh (ViSession instrSession, ViBoolean chan,
    ViInt32 variable, ViPReal64 measVal)
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;

    if (rsupl_invalidViBooleanRange (chan))
        return VI_ERROR_PARAMETER2;
    if (rsupl_invalidViInt32Range (variable, 0, 3))
        return VI_ERROR_PARAMETER3;
        
    if (variable < 3)
        Fmt (buffer, "%s<:SENS%d[b4]:DATA%d[b4]?", variable+1, chan+1);
    else        
        Fmt (buffer, "%s<:SENS%d[b4]:DATA?", variable+1);

    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
        
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%f", measVal) < 1) {
        rsupl_status = VI_ERROR_INTERPRETING_RESPONSE;
        return rsupl_status;
    }

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Read Dual Channel                                               */
/* Purpose:  This VI subsequently reads two measurements from channel 1 and  */
/*           channel 2 of the same analyzer instrument.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_measDualCh (ViSession instrSession, ViInt32 variable,
                                    ViPReal64 measuredValueCh1,
                                    ViPReal64 measuredValueCh2)
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;

    if (rsupl_invalidViInt32Range (variable, 0, 3))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<\n:SENS%d[b4]:DATA1?", variable+1);

    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;
        
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%f", measuredValueCh1) < 1) {
        return VI_ERROR_INTERPRETING_RESPONSE;
    }
    
    Fmt (buffer, "%s<\n:SENS%d[b4]:DATA2?", variable+1);

    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;
        
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%f", measuredValueCh2) < 1) {
        return VI_ERROR_INTERPRETING_RESPONSE;
    }

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Protocol Analysis                                               */
/* Purpose:  This function returns results of protocol analyzes.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_measProtocol (ViSession instrSession, 
                    ViChar _VI_FAR chanStatus[], ViChar _VI_FAR LRChanData[], 
                    ViChar _VI_FAR valBitR[], ViChar _VI_FAR genErr[], 
                    ViChar _VI_FAR valBitL[], ViPInt32 parityErr, ViPInt32 CRCErrL, 
                    ViPInt32 CRCErrR)
{
    ViStatus rsupl_status = VI_SUCCESS;
    
    *parityErr = 0;
    *CRCErrL = 0;
    *CRCErrR = 0;
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:ERR:GEN?", genErr)) < 0)
        return rsupl_status;    
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:CHST?", chanStatus)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:LR?", LRChanData)) < 0)
        return rsupl_status;    

    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:RVAL?", valBitR)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:LVAL?", valBitL)) < 0)
        return rsupl_status;    
        
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:ERR:PAR?", (ViChar *)buffer)) < 0)
        return rsupl_status;    
        
    if (Scan (buffer, "%s>%d[b4]", parityErr) < 0) {
        rsupl_status = VI_ERROR_INTERPRETING_RESPONSE;
        return rsupl_status;
    }
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:ERR:LCRC?", (ViChar *)buffer)) < 0)
        return rsupl_status;    
        
    if (Scan (buffer, "%s>%d[b4]", CRCErrL) < 0) {
        rsupl_status = VI_ERROR_INTERPRETING_RESPONSE;
        return rsupl_status;
    }
    
    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":DISP:PROT:ERR:RCRC?", (ViChar *)buffer)) < 0)
        return rsupl_status;    
        
    if (Scan (buffer, "%s>%d[b4]", CRCErrR) < 0) {
        rsupl_status = VI_ERROR_INTERPRETING_RESPONSE;
        return rsupl_status;
    }
    
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Limit Check                                                     */
/* Purpose:  This function returns 1 when a limit is exceeded, otherwise 0.  */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_measLimit (ViSession instrSession, ViPInt32 measVal)
{
    ViStatus rsupl_status = VI_SUCCESS;

    if ((rsupl_status = rsupl_getInstrResponse (instrSession, ":CALC:LIM:FAIL?", (ViChar *)buffer)) < 0)
        return rsupl_status;    

    if (Scan (buffer, "%s>%d[b4]", measVal) < 0) {
        return VI_ERROR_INTERPRETING_RESPONSE;
    }

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Set Frequency & Level                                           */
/* Purpose:  This VI is useful for setting generator frequency and level,    */
/*           if a sweep measurement is to be controlled by a loop. See       */
/*           configuration functions for more details.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_setFreqLevel (ViSession instrSession,
                                      ViBoolean setFrequency, ViReal64 frequency,
                                      ViInt32 frequencyUnit, ViBoolean setLevel,
                                      ViReal64 level, ViInt32 levelUnit)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 bytesFmt = 0;

    if (rsupl_invalidViBooleanRange (setFrequency))
        return VI_ERROR_PARAMETER2;
    
    if (rsupl_invalidViBooleanRange (setLevel))
        return VI_ERROR_PARAMETER5;
    
    Fmt (buffer, "%s<\n");
    bytesFmt += NumFmtdBytes ();
    if (setFrequency) {
        if (rsupl_invalidViReal64Range (frequency, 0.0, 1.0e6))
            return VI_ERROR_PARAMETER3;
        if (rsupl_invalidViInt32Range (frequencyUnit, 0, 8))
            return VI_ERROR_PARAMETER4;
        Fmt (buffer, "%s<:SOUR:FREQ %f%s", frequency, unitFreqArr[frequencyUnit]);
        bytesFmt += NumFmtdBytes ();
    }
    
    if (setLevel) {
        if (rsupl_invalidViReal64Range (level, -1.0e3, 1.0e6))
            return VI_ERROR_PARAMETER6;
        if (rsupl_invalidViInt32Range (levelUnit, 0, 26))
            return VI_ERROR_PARAMETER7;
        Fmt (buffer, "%s[a]<;:SOUR:VOLT %f%s", level, unitLevelArr[levelUnit]);
        bytesFmt += NumFmtdBytes ();
    }

    if ((rsupl_status = viWrite (instrSession, buffer, bytesFmt, &retCnt)) < 0)
        return rsupl_status;
            
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Send Trigger                                                    */
/* Purpose:  This function triggers an action.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_sendTrig (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;

    if ((rsupl_status = viWrite (instrSession, (ViByte *)":INIT;*WAI", 10, &retCnt)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Step                                                            */
/* Purpose:  This function steps up or down the manual sweep or manual list. */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_step (ViSession instrSession, ViBoolean direction)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    if (rsupl_invalidViBooleanRange (direction))
        return VI_ERROR_PARAMETER2;
        
    Fmt (buffer, "%s<:INIT:NEXT %s", stepDirArr[direction]);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;        
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Abort                                                           */
/* Purpose:  This function aborts an action.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_abort (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    if ((rsupl_status = viWrite (instrSession, (ViByte *)":ABOR", 5, &retCnt)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Store Screen To File                                            */
/* Purpose:  This function stores the screen to a file.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_screenToFile (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    if ((rsupl_status = viWrite (instrSession, (ViByte *)":HCOP:IMM;*WAI", 14, &retCnt)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*=========================================================================*/
/* Function: Operation Complete                                            */
/* Purpose:  This function waits until all previous operations will have   */
/*           been completed.                                               */
/*=========================================================================*/
ViStatus _VI_FUNC rsupl_operationComplete (ViSession instrSession, ViInt32 maxTimeout)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViUInt16 response = 0;
    ViInt32 i = 0;

    if (rsupl_invalidViReal64Range (maxTimeout, 0, 600))
        return VI_ERROR_PARAMETER2;

    if ((rsupl_status = viWrite (instrSession, (ViByte *)"*WAI;*OPC?", 10, &retCnt)) < 0)
        return rsupl_status;

    while (!(response & 0x10)) {
        if ((rsupl_status = viReadSTB (instrSession, &response)) < 0)
            return rsupl_status;

        Delay(0.1);
        i++;
        if (i/10 > maxTimeout) {
            rsupl_status = VI_ERROR_TMO;
            return rsupl_status;
        }
    }

    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    return rsupl_status;
}


/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_writeInstrData (ViSession instrSession, ViString writebuffer)
{
    ViStatus rsupl_status = VI_SUCCESS;

    if ((rsupl_status = viPrintf (instrSession, "%s", writebuffer)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Read Instrument buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_readInstrData (ViSession instrSession, ViInt32 numBytes,
                    ViChar _VI_FAR rdBuf[], ViPInt32 bytesRead)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32    retCnt;

    if (bytesRead)
        *bytesRead = 0L;

    if ((rsupl_status = viRead (instrSession, (ViByte *)rdBuf, numBytes, &retCnt)) < 0)
        return rsupl_status;

    if (bytesRead)
        *bytesRead = (ViInt32)retCnt;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_reset (ViSession instrSession)
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    if ((rsupl_status = viWrite (instrSession, (ViByte *)"*RST", 4, &retCnt)) < 0)
        return rsupl_status;

    if ((rsupl_status = rsupl_defaultInstrSetup (instrSession)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_self_test (ViSession instrSession, ViPInt16 testResult,
    ViChar _VI_FAR selfTestMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;

    if ((rsupl_status = viWrite (instrSession, (ViByte *)"*TST?", 5, &retCnt)) < 0)
        return rsupl_status;
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%d[b2]", testResult) < 0) {
        return VI_ERROR_INTERPRETING_RESPONSE;
    }

    if (*testResult == 0)
        Fmt (selfTestMessage, "Self-test passed.");
    else
        Fmt (selfTestMessage, "Self-test failed.");
        
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_error_query (ViSession instrSession, ViPInt32 errCode,
                    ViChar _VI_FAR errMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;

    if ((rsupl_status = viWrite (instrSession, (ViByte *)":SYST:ERR?", 10, &retCnt)) < 0)
        return rsupl_status;

    if ((rsupl_status = viScanf (instrSession, "%ld,\"%[^\"]", errCode, errMessage)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_error_message (ViSession instrSession, ViStatus errorCode,
                    ViChar _VI_FAR errMessage[])
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViInt32 i;
    static rsupl_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9,   "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_INSTR_PARAMETER16,  "ERROR: Parameter 16 out of range"},
        {VI_ERROR_INSTR_PARAMETER17,  "ERROR: Parameter 17 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_RSUPL_ERROR_INSTRUMENT_ERROR,  "ERROR: Instrument specific error"},
        {VI_RSUPL_ERROR_ANALYZER_FUNCTION, "ERROR: Function not available - instrument setup conflict"},
        {VI_RSUPL_ERROR_BLOCK_DATA,   "ERROR: Block data not available"},
        {VI_NULL, VI_NULL}
    };

    rsupl_status = viStatusDesc (instrSession, errorCode, errMessage);
    if (rsupl_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }

    rsupl_status = VI_SUCCESS;
    return rsupl_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_revision_query (ViSession instrSession,
    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCnt = 0;
    ViStatus rsupl_status = VI_SUCCESS;

    if ((rsupl_status = viWrite (instrSession, (ViByte *)"*IDN?", 5, &retCnt)) < 0)
        return rsupl_status;

    if ((rsupl_status = viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev)) < 0)
        return rsupl_status;

    Fmt (driverRev, "%s<%s", RSUPL_REVISION);

    return rsupl_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsupl_close (ViSession instrSession)
{
    rsupl_instrRange instrPtr;
    ViSession rmSession;
    ViStatus rsupl_status = VI_SUCCESS;

    if ((rsupl_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return rsupl_status;
    if ((rsupl_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rsupl_status;

    if (instrPtr != NULL) 
        free (instrPtr);

    rsupl_status = viClose (instrSession);
    viClose (rmSession);

    return rsupl_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsupl_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return      */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsupl_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsupl_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsupl_init function.  When    */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsupl_init to        */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsupl_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*****************************************************************************/
/*----------- INSTRUMENT-DEPENDENT UTILITY ROUTINES -------------------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rsupl_reset operation and by the      */
/*           rsupl_init function if the reset option has not been            */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rsupl_defaultInstrSetup (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    rsupl_instrRange instrPtr;
   
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if ((rsupl_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return rsupl_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct rsupl_statusDataRanges));

    Fmt (instrPtr -> instrDriverRevision, "%s<%s", RSUPL_REVISION);

    if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)instrPtr) != 0)
        return rsupl_status;

    /*----------- Clear Status Structures ---------------------------------*/
    if ((rsupl_status = viWrite (instrSession, (ViByte *)"*CLS;:FORM:DATA REAL", 20, &retCnt)) < 0)
        return rsupl_status;

    return rsupl_status;
}

/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function reads the Standard Event Register and checks    */
/*           for error.                                                    */
/*=========================================================================*/
ViStatus rsupl_sysStatus (ViSession instrSession)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 esr;

    /*--------------- Read the Standard Event Register --------------------*/
    if ((rsupl_status = viWrite (instrSession, (ViByte *)"*ESR?", 5, &retCnt)) < 0)
        return rsupl_status;
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%d[b4]", &esr) != 1) {
        rsupl_status = VI_ERROR_INTERPRETING_RESPONSE;
        return rsupl_status;
    }

    /*---------------- Check if any error bit was asserted ----------------*/
    if ((esr & 0x3C) != 0)
        rsupl_status = VI_RSUPL_ERROR_INSTRUMENT_ERROR;
    else
       rsupl_status = VI_SUCCESS;

    return rsupl_status;
}

/*=========================================================================*/
/* Function: Get Digital Generator Mode                                    */
/* Purpose:  This function returns the instrument type & digital mode.     */
/*           InstrType: 0 - A25, 1 - D48                                   */
/*           Mode: 0 - Jitt/Ph/Comm, 1 - Audio Data                        */
/*=========================================================================*/
ViStatus rsupl_getGenMode (ViSession instrSession, ViPBoolean InstrType,
                                                   ViPBoolean Mode)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViInt32 index = -1, i = 0;

    if ((rsupl_status = viWrite (instrSession, (ViByte *)"\n:INST:NSEL?\n", 13, &retCnt)) < 0)
        return rsupl_status;

    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;
    
    if (Scan (buffer, "%s>%d[b4]", &i) != 1)
        return VI_ERROR_INTERPRETING_RESPONSE;
    
    *InstrType = (ViBoolean)(i == 3);
    if (*InstrType) {
        if ((rsupl_status = viWrite (instrSession, (ViByte *)"\n:SOUR:DIG:FEED?\n", 17, &retCnt)) < 0)
            return rsupl_status;        

        if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rsupl_status;
        
        index = FindPattern ((ViChar *)buffer, 0, -1, "ADAT", 0, 0);
        *Mode = (ViBoolean)((index == -1) ? VI_FALSE: VI_TRUE);
    } else {
        *Mode = VI_TRUE;
    }
    
    return rsupl_status;
}        

/*=========================================================================*/
/* Function: Get Analyzer Mode                                             */
/* Purpose:  This function returns the instrument analyzer mode.           */
/*          instrType = VI_TRUE;  Digital Analyzer                         */
/*          instrType = VI_FALSE; Analog Analyzer                          */
/*          *digMode = 0;  Digital Analyzer - Meas. Mode Audio Data        */
/*          *digMode = 1;  Digital Analyzer - Meas. Mode Jitter            */
/*          *digMode = 2;  Digital Analyzer - Meas. Mode Common/Inp        */
/*=========================================================================*/
ViStatus rsupl_getAnalyzerMode (ViSession instrSession, ViPInt32 digMode,
    ViPBoolean instrType)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViChar response[50];
    
    if ((rsupl_status = viWrite (instrSession, (ViByte *)":INST2:SEL?", 11, &retCnt)) < 0)
        return rsupl_status;        
        
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%s", response) < 0)
        return VI_ERROR_INTERPRETING_RESPONSE;

    if (FindPattern (response, 0, -1, "A", 0, 0) < 0)
        *instrType = VI_TRUE;  /* Digital Analyzer */
    else        
        *instrType = VI_FALSE; /* Analog Analyzer */
    
    if (*instrType == VI_TRUE) {
        if ((rsupl_status = viWrite (instrSession, (ViByte *)":SENS:DIG:FEED?", 15, &retCnt)) < 0)
            return rsupl_status;        
        
        if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
            return rsupl_status;

        if (Scan (buffer, "%s>%s", response) < 0)
            return VI_ERROR_INTERPRETING_RESPONSE;

        *digMode = 0;

        if (FindPattern (response, 0, -1, "ADAT", 0, 0) >= 0)
            *digMode = 0;  /* Digital Analyzer - Meas. Mode Audio Data */
        if (FindPattern (response, 0, -1, "JPH", 0, 0) >= 0)
            *digMode = 1;  /* Digital Analyzer - Meas. Mode Jitter */
        if (FindPattern (response, 0, -1, "CINP", 0, 0) >= 0)
            *digMode = 2;  /* Digital Analyzer - Meas. Mode Common/Inp */
    }       
        
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}

/*=========================================================================*/
/* Function: Get Active Channels                                           */
/* Purpose:  This function returns the current active channels.            */
/*          chanType = 1;   Channel 1                                      */
/*          chanType = 2;   Channel 2                                      */
/*          chanType = 3;   Channels 1 & 2                                 */
/*=========================================================================*/
ViStatus rsupl_getAnalyzerActiveChannels (ViSession instrSession, ViPInt32 chanType)
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViChar response[50];
    
    if ((rsupl_status = viWrite (instrSession, (ViByte *)":INP:SEL?", 9, &retCnt)) < 0)
        return rsupl_status;        
        
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%s", response) < 0) {
        rsupl_status = VI_ERROR_INTERPRETING_RESPONSE;
        return rsupl_status;
    }
    
    if (NumFmtdBytes () == 3) {
        if (FindPattern (response, 0, -1, "1", 0, 0) < 0)
            *chanType = 2; /* Channel 2 active */
        else        
            *chanType = 1; /* Channel 1 active */
    }
    else
        *chanType = 3;    /* Channels 1 & 2 active */
        
    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}
            
/*=========================================================================*/
/* Function: Get Instrument Response                                       */
/* Purpose:  This function queries the instrument and reads out the        */
/*           response.                                                     */
/*=========================================================================*/
ViStatus rsupl_getInstrResponse (ViSession instrSession, ViChar _VI_FAR query[], 
    ViChar _VI_FAR resp[])
{
    ViStatus rsupl_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    
    Fmt (buffer, "%s<%s", query);
    if ((rsupl_status = viWrite (instrSession, buffer, NumFmtdBytes (), &retCnt)) < 0)
        return rsupl_status;
    
    FillBytes ((ViChar *)buffer, 0, 1024, 0x0);
        
    if ((rsupl_status = viRead (instrSession, buffer, BUFFER_SIZE, &retCnt)) < 0)
        return rsupl_status;

    if (Scan (buffer, "%s>%s", resp) < 0) {
        return VI_ERROR_INTERPRETING_RESPONSE;
    }

    if ((rsupl_status = rsupl_sysStatus (instrSession)) < 0)
        return rsupl_status;
        
    return rsupl_status;
}
            
/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
